\ ==============================================================
\ [TITLE]     8051-ANS Forth 8051-IO word set. 
\ [FILE]      8051-IO.FRT
\ [CATEGORY]  Example 
\ [VSN]       1.00 
\ [CREATED]   January 12, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] october 20, 1994, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      ANS-Forth 
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   An example of storing and fetching from the 
\             external I/O-range (use of MOVX) in the ROM
\             address range. It is also an other example of using
\             the METHODS mechanism and the use of assembler.
\ ==============================================================

ALSO EXTRA  DEFINITIONS

MARKER -8051-IO 

\ Define TO-operators for DATA

ONLY  EXTRA ALSO  FORTH ALSO  INTERNAL ALSO  DEFINITIONS

CODE TO-DATA  ( byte inline-adr -- )
    DPH:                POP,            \ Get address of inline address
    DPL:                POP,

    A:                  CLR,            \ Reset offset
    A:      @A+DPTR    MOVC,            \ Fetch inline address
    R2:     A:          MOV,            \ Save highbyte
    DPTR:               INC,
    A:                  CLR,            \ Reset offset
    A:      @A+DPTR    MOVC,            \ Get lowbyte

    DPTR:               INC,            \ Push new returnaddress
    DPL:               PUSH,
    DPH:               PUSH,
    
    DPL:    A:          MOV,            \ Set pointer to data address
    DPH:    R2:         MOV,
    
    R0:                 INC,            \ Pop highbyte stack
    A:      @R0        MOVX,            \ Fetch lowbyte from stack
    @DPTR   A:         MOVX,            \ Store in DATA address
    R0:                 INC,            \ Pop lowbyte
                        RET,
END-CODE  C/O  TAIL

CODE ADR-DATA  ( inline-adr -- data-adr )
    DPH:                POP,            \ Get address of inline address
    DPL:                POP,

    A:                  CLR,            \ Reset offset
    A:      @A+DPTR    MOVC,            \ Fetch inline address high
    R2:     A:          MOV,            \ Save highbyte
    A:      1 #         MOV,            \ Reset offset
    A:      @A+DPTR    MOVC,            \ Get lowbyte

    R0:                 DEC,            \ Push lowbyte to stack
    @R0     A:         MOVX,
    R0:                 DEC,            \ Push lowbyte to stack
    A:      R2:         MOV,            \ Get highbyte
    @R0     A:         MOVX,            \ Push highbyte to stack
    
    A:      2 #         MOV,            \ Offset to code
    @A+DPTR             JMP,            \ Jump to that code
END-CODE  C/O  TAIL

CODE FROM-DATA  ( offset inline-adr -- byte )
    DPH:                POP,            \ Get address of inline address
    DPL:                POP,

    A:                  CLR,            \ Reset offset
    A:      @A+DPTR    MOVC,            \ Fetch inline address
    R2:     A:          MOV,            \ Save highbyte
    DPTR:               INC,
    A:                  CLR,            \ Reset offset
    A:      @A+DPTR    MOVC,            \ Get lowbyte
    R3:     A:          MOV,            \ Save lowbyte

    DPTR:               INC,            \ Push new returnaddress
    DPL:               PUSH,
    DPH:               PUSH,
    
    R0:                 INC,            \ Pop highbyte stack
    A:      @R0        MOVX,            \ Fetch lowbyte offset from stack
    A:      R3:         ADD,            \ Add DATA-adr to it
    DPL:    A:          MOV,            \ Set DATA-adr low
    R0:                 DEC,            \ To highbyte
    A:      @R0        MOVX,            \ Fetch highbyte offset from stack
    A:      R2:        ADDC,            \ Add DATA-adr to it
    DPH:    A:          MOV,            \ Set DATA-adr high
    ACC: .7 BS           IF,            \ Out of DATA-range
        ]  1 THROW  [                   \ Invalid DATA-adr
                       THEN,
    R0:                 INC,            \ Pop highbyte
    R0:                 INC,            \ Pop lowbyte

\   A:                  CLR,            \ Read from code memory, for testing
\   A:      @A+DPTR    MOVC,

( ) A:      @DPTR      MOVX,            \ Fetch data byte
( Push byte )
    R0:                 DEC,            \ Make room on stack
    @R0     A:         MOVX,            \ Put data on stack
    A:                  CLR,            \ Byte data so highbyte is zero
    R0:                 DEC,            \ Make room for highbyte
    @R0     A:         MOVX,            \ Put data on stack
                        RET,            \ Done, return.
END-CODE  C/O  TAIL

DOERCODE DO-DATA    ( -- byte )
    DPH:                POP,            \ Get address of inline address
    DPL:                POP,

    A:                  CLR,            \ Fetch highbyte first
    A:      @A+DPTR    MOVC,            \ Fetch inline address
    R2:     A:          MOV,            \ Save highbyte of data address
    A:      1 #         MOV,
    A:      @A+DPTR    MOVC,            \ Get lowbyte of data address

    DPL:    A:          MOV,            \ Set pointer to databyte
    DPH:    R2:         MOV,
 
\   A:                  CLR,            \ Read from code memory, for testing
\   A:      @A+DPTR    MOVC,
    
( ) A:      @DPTR      MOVX,            \ Fetch databyte
( Push byte )
    R0:                 DEC,            \ Make room on stack
    @R0     A:         MOVX,            \ Put data on stack
    A:                  CLR,            \ Byte data so highbyte is zero
    R0:                 DEC,            \ Make room for highbyte
    @R0     A:         MOVX,            \ Put data on stack
                        RET,            \ Done, return.
END-CODE  C/O  TAIL

\ We could define separate operators for byte data-elements, 
\ cell data-elements and byte-array data-elements. This version 
\ takes nothing from the stack, so we must define a DATA operator fo each
\ address. It is a good idea to define special prefixes for the data word.
\ So we could add an prefix which takes an extra offset from the stack, and
\ DATA is accessable as an array. Examples of possible names and usage:

\   100 DATA ADC                        \ In/out base address
\
\   ADC                                 \ Give contents of data-address 100.
\   8 TO ADC                            \ Put <8> at address 100
\
\   5 FROM ADC                          \ Give contents address 105
\   25 6 INTO ADC                       \ Put 25 at address 106
\
\   
\   120 DATA BITS                       \ Use as 16 bits i/o address
\
\   FROM-WORD BITS                      \ Give 16 bitscontents of adres 120
\   -1 TO-WORD BITS                     \ Put -1 at 16 bits address 120

EXTRA DEFINITIONS

\G ( c-addr "name" -- )
\G Skip leading space delimiters. Parse name delimited by a space.
\G Create a definition for name with the execution semantics 
\G defined below.
\G 
\G name Execution: ( -- byte )
\G Leave the contents of c-addr on the stack. The contents of
\G c-addr may be changed by using the prefix TO . When the prefix
\G FROM is used, the data space is accessed as an array with
\G c-addr as a starting point.
: DATA                                  \ 8051-IO 
    DUP 0< 1 ?ERROR                     \ Check address I/O only
    CREATE  ,       ( c-addr "name" -- ) \ Create data entry
    DO-DATA         ( -- x )            \ Install execution-interpreter
    ;

METHODS DO-DATA

: TO    ( body -- ) ( value ccc -- )
    POSTPONE TO-DATA   @,               \ Compile data or i/o-function
    ;

: ADR   ( body -- ) ( ccc -- c-addr )
    POSTPONE ADR-DATA  @,               \ Compile data or i/o-function
    ;

: FROM  ( body -- ) ( index ccc -- byte )
    POSTPONE FROM-DATA @,               \ Compile data or i/o-function
    ;

END-METHODS

1 MESS" Invalid data or i/o address"    \ This is an invalid i/o address

ONLY  EXTRA ALSO  FORTH ALSO  DEFINITIONS

                    \ (* End of Source *) /
