\ ===========================================================
\ [TITLE]     8052-ANS-Forth 8052 disassembler words.
\ [FILE]      8052-DIS.FRT
\ [CATEGORY]  Utilities
\ [VSN]       1.52
\ [CREATED]   March 5, 1993, Willem Ouwerkerk 
\ [LASTCHNGD] August 5, 1998, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1993-1998
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8052-ANS-Forth
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Users words for the 8052 disassembler
\       Start disassenbler : <address> DISASSEMBLE
\       Disassemble "name" : DIS "name"
\       Hexdump on/off ... : TRUE TO HEX? or FALSE TO HEX?
\ ==============================================================

?DEF -8052DIS  [IF]  -8052DIS  [THEN]
MARKER -8052DIS     (  8052 disassembler   Version 1.50  )

\ History of 8052 Disassembler
\
\       -- Disassembler voor MCS 51 reeks ( 8052 etc. ) --
\
\ Version 1.50  -- Changed to load above 80C535 target.
\ Version 1.00  -- Working: Saturday 6 march 16.45.. Willem O.

VOCABULARY INSIDE

ONLY  FORTH ALSO  EXTRA ALSO  INTERNAL ALSO  INSIDE ALSO  DEFINITIONS 

0 VALUE HOR
0 VALUE ADDRESS
0 VALUE OPC-ADDR

: INC-ADDRESS     +TO ADDRESS ;
: ADDRESS+1       1 INC-ADDRESS ;
: ADDRESS+2       2 INC-ADDRESS ;
: ADDRESS-1      -1 INC-ADDRESS ;

: ADDR-BITS \ -- n
            OPC-ADDR C@ $F AND  ( Changed ADDRESS to OPC-ADDR saturday 10 mrch)
            ;

: BS            BS          -1 +TO HOR ;
: EMIT          EMIT         1 +TO HOR ;
: TYPE          TUCK TYPE      +TO HOR ;
: CR            CR           0  TO HOR ;
: SPACE         SPACE        1 +TO HOR ;
: SPACES        DUP SPACES     +TO HOR ;
: .HEX          &$ EMIT .HEX 5 +TO HOR ;
: .NAME         COUNT TYPE SPACE ;

: .WORDNAME     ( adr -- )
        DUP >NAME NAME?                 \ Is there a valid header ?
        IF      >NAME .NAME             \ Yes, type name
        ELSE    .HEX                    \ No, print address
        THEN
        ;

: .C        ( d r -- )                  \ Print d in a column r . 
        <#  0 ?DO  #  LOOP  #> TYPE
        ;

: .$"       \ --
            FLYER  POSTPONE S"  POSTPONE TYPE
            ;  IMMEDIATE

: (.BYT     \ x --
            BASE @ >R  HEX  0 2 .C  R> BASE !
            SPACE
            ;

: .BYT        \ x --
            .$" $" (.BYT
            ;

\ Set hard TAB points for disassembler display

: TAB       HOR - SPACES ;
: TAB1      7  TAB ;

: TAB2/3    HOR 27 <
            IF    HOR 17 >  IF  27  ELSE  17  THEN  TAB
            THEN
            ;  

: TAB4      34 TAB ;                    \ *WO*

: .COLUMN   \ c-addr u -- c-addr u
            TAB4                        \ End mode column.
            6 OVER $7 AND - SPACES      \ Fill not needed mnenomic column.
            ;  

\ Various address printing routines for 8052 disassembler

FORTH DEFINITIONS

\G If this value is true a the opcode is also displayed as hexdump
\G the disassembly. when false no hexdump will be displayed.
TRUE VALUE HEX?                         \ View   False is hexdump uit.

INSIDE DEFINITIONS

: .BYTES     \ n --
            HEX?                            \ Show hexdump ?
            IF  3 SPACES  DUP 12 UMIN  0    \ Yes, its for a 80 char. display
                ?DO  OPC-ADDR I + C@ (.BYT  \ Print hexdump
                LOOP
            THEN
            DROP                            \ CR  0 TO HOR \ To new line. *WO*
            ;

: .A-ADDR   \ --
            ADDRESS C@+ $E0 AND  3 LSHIFT   \ Get high 3 bits from opcode.
            SWAP C@+                        \ Low addressbyte to stack.
            SWAP $F800 AND                  \ Page bits to stack.
            OR  OR                          \ Compose address.
            .WORDNAME                       \ Print name or address
            ADDRESS+2                       \ Advance address
            ;                               \ Increase address counter.

: .REL-ADDR \ --
            ADDRESS C@+  DUP 127 >          \ Get relative jump
                                            \ Is it negative ?
            IF  -256 OR  THEN               \ Yes, extend sign and use it
            SWAP +                          \ to increase current address
\           .WORDNAME                       \ Print name or address
            .HEX                            \ No, print address
             ADDRESS+1                      \ Print and adjust address
            ;

: .ADDRESS  \ --
            ADDRESS .HEX  TAB1              \ Print current opcode address
            ADDRESS  TO OPC-ADDR            \ Save this in OPC-ADDR
            ;

: .JUMP     \ --                            \ Print jumpaddress.
            ADDRESS @  .WORDNAME            \ Print name or address
            TAB2/3  ADDRESS+2               \ Advance ...
            ;

\ Reading and printing of strings and numbers for disassembler.

: .OPC      \ c-addr u --
            .COLUMN  TYPE               \ Print opcode in a column.
            ;

: .OPCODE   \ body-adr --
            BODY> >NAME COUNT .COLUMN TYPE  \ Print opc. in a column.
            ;

: .OPCODE"  \ ccc --
            FLYER
            POSTPONE S"  POSTPONE .OPC  \ Print opc-string in a column.
            ;  IMMEDIATE

: .STRING   \ c-addr u --
            TYPE  TAB2/3                    \ Print $ and advance to next TAB
            ;

: .STRING"  \ --
            FLYER
            POSTPONE S"  POSTPONE .STRING   \ Print $, and increase TAB-pos.
            ;  IMMEDIATE

: .REG      \ --
            .$" R"   ADDR-BITS $7 AND       \ Get reg. bits
            0 1 .C .$" :"                   \ Print reg-naam 
            TAB2/3  ADDRESS+1               \ Increase TAB and address
            ;

: .@REG     \ --
            .$" @R"  ADDR-BITS $1 AND       \ Get reg. bits
            0 1 .C                          \ Print ind-reg-name and
            TAB2/3  ADDRESS+1               \ increase TAB and address.
            ;

: DECODE-REG    \ --
            CASE  ADDR-BITS
              $6 OF  .@REG  ENDOF
              $7 OF  .@REG  ENDOF
                     .REG
            ENDCASE
            ;

: .IMM      \ --
            ADDRESS C@ .BYT .$" #"  ADDRESS+1  TAB2/3
            ;

: .ACCU     \ --
            .STRING" A:"
            ;

: .CARRY    \ --
            .STRING" C:"
            ;

: .OTHER    \ addr --
            DUP 0 8 WITHIN              \ Is it a direct register address ?
            IF    .$" [R" 0 1 .C .$" ]" \ Yes, print register name
                  EXIT
            THEN  .BYT
            ;

\ Print only 8052 SFR names as string others as address

: .SFR      \ addr --
            CASE
              $80 OF  .$" P0"      ENDOF    
              $81 OF  .$" SP"      ENDOF    
              $82 OF  .$" DPL"     ENDOF    
              $83 OF  .$" DPH"     ENDOF    
              $87 OF  .$" PCON"    ENDOF    
              $88 OF  .$" TCON"    ENDOF    
              $89 OF  .$" TMOD"    ENDOF    
              $8A OF  .$" TL0"     ENDOF    
              $8B OF  .$" TL1"     ENDOF    
              $8C OF  .$" TH0"     ENDOF    
              $8D OF  .$" TH1"     ENDOF    
              $90 OF  .$" P1"      ENDOF    
              $98 OF  .$" SCON"    ENDOF    
              $99 OF  .$" SBUF"    ENDOF    
              $A0 OF  .$" P2"      ENDOF    
              $A8 OF  .$" IE"      ENDOF    
              $B0 OF  .$" P3"      ENDOF    
              $B8 OF  .$" IP"      ENDOF    
              $C8 OF  .$" T2CON"   ENDOF    
              $C9 OF  .$" T2MOD"   ENDOF    
              $CA OF  .$" RCAP2L"  ENDOF    
              $CB OF  .$" RCAP2H"  ENDOF    
              $CC OF  .$" TL2"     ENDOF    
              $CD OF  .$" TH2"     ENDOF    
              $D0 OF  .$" PSW"     ENDOF    
              $E0 OF  .$" ACC"     ENDOF    
              $F0 OF  .$" B"       ENDOF
                      .OTHER  EXIT
            ENDCASE
            .$" :"  
            ;
            
: .BYTE      \ --
            ADDRESS C@ .SFR   TAB2/3  ADDRESS+1
            ;

: .BIT      \ flag --
            ADDR-BITS 2 AND 0=  AND     \ Inverted bit ?
            IF .$" /"  THEN             \ Yes, print '/'
            ADDRESS C@ DUP 7 AND >R     \ Save bitnumber.
            DUP $80 <                   \ Not a SFR ?
            IF    3 RSHIFT  $20 +       \ Make bitaddress   
            ELSE  R@ -                  \ Make bare SFR address.
            THEN
            .SFR  BS                    \ Print SFR-name.
            .$" ."  R> 0 1 .C           \ Pint bitnumber and
            TAB2/3  ADDRESS+1           \ to next TAB etc.
            ;

\ Building words for disassembler

: BUILD$    \ --
            &" PARSE 2DROP  &" WORD
            C@ 1+ ALLOT  ALIGN
            ;
            
\ Instructions with only one address mode, 'simple to translate'.

: 1BYTE     CREATE  ,  BUILD$
            DOES>   .ADDRESS  DUP @ 
                    IF  DUP CELL+ COUNT .STRING  THEN  
                    ADDRESS+1  .OPCODE  1 .BYTES
            ;

: 2BYTES    CREATE
            DOES>   .ADDRESS  ADDRESS+1  .BYTE  .OPCODE  2 .BYTES
            ;

: JUMPS     CREATE
            DOES>   .ADDRESS  ADDRESS+1  .JUMP  .OPCODE  3 .BYTES
                    ;

: AJUMPS    CREATE
            DOES>   .ADDRESS  .A-ADDR  .OPCODE  2 .BYTES
                    ;

: BRANCH    CREATE  ,
            DOES>   .ADDRESS  ADDRESS+1
                    DUP @ IF  0 .BIT  THEN
                    .REL-ADDR  DUP .OPCODE
                    2 SWAP @ IF  1+  THEN  .BYTES
                    ;

: 3SMODES   CREATE
            DOES>   .ADDRESS  ADDRESS+1  ADDR-BITS
                    CASE
                      $2 OF  FALSE .BIT ENDOF
                      $3 OF  .CARRY     ENDOF
                      $4 OF  .ACCU      ENDOF
                    ENDCASE
                    .OPCODE  1  ADDR-BITS $7 AND $2 =
                    IF  1+  THEN  .BYTES
                    ;

: 4SMODES   CREATE  ,
            DOES>   .ADDRESS  1  OVER @ IF  1+  THEN
                    CASE  ADDR-BITS
                      $3 OF  ADDRESS+1  .STRING" DPTR:" ENDOF
                      $4 OF  ADDRESS+1  .ACCU           ENDOF
                      $5 OF  ADDRESS+1  .BYTE  1+       ENDOF
                             DECODE-REG
                    ENDCASE
                    OVER @ IF  .REL-ADDR  THEN  SWAP .OPCODE  .BYTES
                    ;

\ Instructions with one or two addressing modes.

: 4MODES    CREATE
            DOES>   .ADDRESS  1
                    CASE  ADDR-BITS  .ACCU
                      $4 OF  ADDRESS+1  .IMM   1+  ENDOF
                      $5 OF  ADDRESS+1  .BYTE  1+  ENDOF
                             DECODE-REG
                    ENDCASE
                    SWAP .OPCODE  .BYTES
                    ;

: 6MODES    CREATE
            DOES>   .ADDRESS  1
                    CASE  ADDR-BITS
                      $2 OF  ADDRESS+1  .BYTE  .ACCU  1+  ENDOF
                      $3 OF  ADDRESS+1  .BYTE  .IMM   2 + ENDOF
                      $4 OF  ADDRESS+1  .ACCU  .IMM   1+  ENDOF
                      $5 OF  ADDRESS+1  .ACCU  .BYTE  1+  ENDOF
                             .ACCU  DECODE-REG
                    ENDCASE  
                    SWAP .OPCODE  .BYTES  
                    ;

\ Disassembler action words.
 
0   1BYTE NOP,
1   1BYTE RR,      "A:"
0   1BYTE RET,
0   1BYTE RETI,
1   1BYTE RL,      "A:"
1   1BYTE RLC,     "A:"
1   1BYTE DIV,     "AB:"
1   1BYTE RRC,     "A:"
1   1BYTE ???,     "Invalid opcode: $A5"
1   1BYTE MUL,     "AB:"
1   1BYTE SWAP,    "A:"
1   1BYTE DA,      "A:"
1   1BYTE JMP,     "@A+DPTR"
 
   2BYTES PUSH,
   2BYTES POP,

  3SMODES CPL,
  3SMODES CLR,
  3SMODES SETB,

0 4SMODES INC,
0 4SMODES DEC,
1 4SMODES DJNZ,

   4MODES ADD,
   4MODES ADDC,
   4MODES SUBB,
   4MODES XCH,
   4MODES XCHD,

   6MODES ORL,
   6MODES ANL,
   6MODES XRL,

: OR-BIT    \ --
            .ADDRESS  ADDRESS+1  .CARRY  TRUE .BIT
            .OPCODE" ORL,"  2 .BYTES
            ;

: AND-BIT   \ --
            .ADDRESS  ADDRESS+1  .CARRY  TRUE .BIT  
            .OPCODE" ANL,"  2 .BYTES
            ;

   JUMPS LJMP,
   JUMPS LCALL,
  AJUMPS AJMP,
  AJUMPS ACALL,

0 BRANCH JC,
0 BRANCH JNC,
0 BRANCH JZ,
0 BRANCH JNZ,
0 BRANCH SJMP,

1 BRANCH JBC,
1 BRANCH JB,
1 BRANCH JNB,

\ Docode conditional jumps

: DECODE-CJNE   \ -- n
            2                                           \ *WO*
            CASE  ADDR-BITS
              $4 OF  ADDRESS+1  .ACCU             ENDOF \ *WO*
              $5 OF  ADDRESS+1  .ACCU  .BYTE EXIT ENDOF \ *WO*
                     DECODE-REG          
            ENDCASE
            .IMM
            ;

: CJNE      \ -- n
            .ADDRESS  DECODE-CJNE  
            .REL-ADDR  1+  .OPCODE" CJNE,"  .BYTES
            ;

: MOVX,     \ --
            .ADDRESS
            CASE  ADDRESS C@ 
              $E0 OF  .ACCU  .STRING" @DPTR"  ADDRESS+1 ENDOF
              $E2 OF  .ACCU  .@REG                      ENDOF
              $E3 OF  .ACCU  .@REG                      ENDOF
              $F0 OF  .STRING" @DPTR"  .ACCU  ADDRESS+1 ENDOF
              $F2 OF  .@REG  .ACCU                      ENDOF
              $F3 OF  .@REG  .ACCU                      ENDOF
            ENDCASE
            .OPCODE" MOVX,"  1 .BYTES
            ;

: MOVC,     \ --
            .ADDRESS  .ACCU  ADDRESS C@ $10 AND
            IF  .STRING" @A+DPTR"  ELSE  .STRING" @A+PC"  THEN
            ADDRESS+1  .OPCODE" MOVC,"  1 .BYTES
            ;

\ And now the large mass, the disassembling of MOV,

: DECODE-IMM    \ -- n
            2                                           \ *WO*
            CASE  ADDR-BITS
              $4 OF  ADDRESS+1  .ACCU             ENDOF \ *WO*
              $5 OF  ADDRESS+1  .BYTE  1+         ENDOF \ *WO*
                     DECODE-REG          
            ENDCASE
            .IMM
            ;

: MOVE-TO-ADDR  \ -- n
            2  ADDR-BITS $5 = IF  1+  THEN
            CASE  ADDR-BITS  ADDRESS+1  .BYTE
              $5 OF  .BYTE              ENDOF
                     DECODE-REG  ADDRESS-1
            ENDCASE
            ;

: MOVE-FROM-ADDR    \ -- n
            DECODE-REG  .BYTE   2
            ;

: MOVE-TO-ACCU      \ -- n
            .ACCU  ADDRESS C@ $E5 =
            IF    ADDRESS+1  .BYTE  2
            ELSE  DECODE-REG  1
            THEN
            ;

: MOVE-FROM-ACCU    \ -- n
            ADDRESS C@ $F5 =
            IF    ADDRESS+1  .BYTE  2
            ELSE  DECODE-REG  1  
            THEN
            .ACCU
            ;

: MOVE-DPTR \ --
            .STRING" DPTR:"  ADDRESS @ .HEX 
            .STRING"  #"  ADDRESS+2
            ;

: OTHER-MOVES   \ -- n
            2  ADDRESS C@ $90 = IF  1+  THEN
            CASE  ADDRESS C@  ADDRESS+1
              $90 OF  MOVE-DPTR             ENDOF
              $92 OF  FALSE .BIT  .CARRY    ENDOF
              $A2 OF  .CARRY  FALSE .BIT    ENDOF
              $E5 OF  .ACCU  .BYTE          ENDOF
              $F5 OF  .BYTE  .ACCU          ENDOF
            ENDCASE
            ; 
 
: MOV,      \ --
            .ADDRESS  OPC-ADDR ( *WO* ) C@ $74 $80 WITHIN
            IF    DECODE-IMM
            ELSE  ADDRESS C@ $85 $90 WITHIN
                  IF    MOVE-TO-ADDR
                  ELSE  ADDRESS C@ $A6 $B0 WITHIN
                        IF    MOVE-FROM-ADDR
                        ELSE  ADDRESS C@ $E5 $F0 WITHIN
                              IF    MOVE-TO-ACCU
                              ELSE  ADDRESS C@ $F5 $100 WITHIN
                                    IF    MOVE-FROM-ACCU
                                    ELSE  OTHER-MOVES
                                    THEN
                              THEN
                        THEN
                  THEN
            THEN
            .OPCODE" MOV,"  .BYTES
            ;

\ The translate table, for each opcode a cell with a token.

CREATE OPCODE-TABLE

( 00 ) ' NOP,    ,   ' AJMP,   ,   ' LJMP,   ,   ' RR,     ,   ' INC,    ,
( 05 ) ' INC,    ,   ' INC,    ,   ' INC,    ,   ' INC,    ,   ' INC,    ,
( 0A ) ' INC,    ,   ' INC,    ,   ' INC,    ,   ' INC,    ,   ' INC,    ,
( 0F ) ' INC,    ,   ' JBC,    ,   ' ACALL,  ,   ' LCALL,  ,   ' RRC,    ,
( 14 ) ' DEC,    ,   ' DEC,    ,   ' DEC,    ,   ' DEC,    ,   ' DEC,    ,
( 19 ) ' DEC,    ,   ' DEC,    ,   ' DEC,    ,   ' DEC,    ,   ' DEC,    ,
( 1E ) ' DEC,    ,   ' DEC,    ,   ' JB,     ,   ' AJMP,   ,   ' RET,    ,
( 23 ) ' RL,     ,   ' ADD,    ,   ' ADD,    ,   ' ADD,    ,   ' ADD,    ,
( 28 ) ' ADD,    ,   ' ADD,    ,   ' ADD,    ,   ' ADD,    ,   ' ADD,    ,
( 2D ) ' ADD,    ,   ' ADD,    ,   ' ADD,    ,   ' JNB,    ,   ' ACALL,  ,
( 32 ) ' RETI,   ,   ' RLC,    ,   ' ADDC,   ,   ' ADDC,   ,   ' ADDC,   ,
( 37 ) ' ADDC,   ,   ' ADDC,   ,   ' ADDC,   ,   ' ADDC,   ,   ' ADDC,   ,
( 3C ) ' ADDC,   ,   ' ADDC,   ,   ' ADDC,   ,   ' ADDC,   ,   ' JC,     ,
( 41 ) ' AJMP,   ,   ' ORL,    ,   ' ORL,    ,   ' ORL,    ,   ' ORL,    ,
( 46 ) ' ORL,    ,   ' ORL,    ,   ' ORL,    ,   ' ORL,    ,   ' ORL,    ,
( 4B ) ' ORL,    ,   ' ORL,    ,   ' ORL,    ,   ' ORL,    ,   ' ORL,    ,
( 50 ) ' JNC,    ,   ' ACALL,  ,   ' ANL,    ,   ' ANL,    ,   ' ANL,    ,
( 55 ) ' ANL,    ,   ' ANL,    ,   ' ANL,    ,   ' ANL,    ,   ' ANL,    ,
( 5A ) ' ANL,    ,   ' ANL,    ,   ' ANL,    ,   ' ANL,    ,   ' ANL,    ,
( 5F ) ' ANL,    ,   ' JZ,     ,   ' AJMP,   ,   ' XRL,    ,   ' XRL,    ,
( 64 ) ' XRL,    ,   ' XRL,    ,   ' XRL,    ,   ' XRL,    ,   ' XRL,    ,
( 69 ) ' XRL,    ,   ' XRL,    ,   ' XRL,    ,   ' XRL,    ,   ' XRL,    ,
( 6E ) ' XRL,    ,   ' XRL,    ,   ' JNZ,    ,   ' ACALL,  ,   ' OR-BIT  ,
( 73 ) ' JMP,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,
( 78 ) ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,
( 7D ) ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' SJMP,   ,   ' AJMP,   ,
( 82 ) ' AND-BIT ,   ' MOVC,   ,   ' DIV,    ,   ' MOV,    ,   ' MOV,    ,
( 87 ) ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,
( 8C ) ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,
( 91 ) ' ACALL,  ,   ' MOV,    ,   ' MOVC,   ,   ' SUBB,   ,   ' SUBB,   ,
( 96 ) ' SUBB,   ,   ' SUBB,   ,   ' SUBB,   ,   ' SUBB,   ,   ' SUBB,   ,
( 9B ) ' SUBB,   ,   ' SUBB,   ,   ' SUBB,   ,   ' SUBB,   ,   ' SUBB,   ,
( A0 ) ' OR-BIT  ,   ' AJMP,   ,   ' MOV,    ,   ' INC,    ,   ' MUL,    ,
( A5 ) ' ???,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,
( AA ) ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,
( AF ) ' MOV,    ,   ' AND-BIT ,   ' ACALL,  ,   ' CPL,    ,   ' CPL,    ,
( B4 ) ' CJNE    ,   ' CJNE    ,   ' CJNE    ,   ' CJNE    ,   ' CJNE    ,
( B9 ) ' CJNE    ,   ' CJNE    ,   ' CJNE    ,   ' CJNE    ,   ' CJNE    ,
( BE ) ' CJNE    ,   ' CJNE    ,   ' PUSH,   ,   ' AJMP,   ,   ' CLR,    ,
( C3 ) ' CLR,    ,   ' SWAP,   ,   ' XCH,    ,   ' XCH,    ,   ' XCH,    ,
( C8 ) ' XCH,    ,   ' XCH,    ,   ' XCH,    ,   ' XCH,    ,   ' XCH,    ,
( CD ) ' XCH,    ,   ' XCH,    ,   ' XCH,    ,   ' POP,    ,   ' ACALL,  ,
( D2 ) ' SETB,   ,   ' SETB,   ,   ' DA,     ,   ' DJNZ,   ,   ' XCHD,   ,
( D7 ) ' XCHD,   ,   ' DJNZ,   ,   ' DJNZ,   ,   ' DJNZ,   ,   ' DJNZ,   ,
( DC ) ' DJNZ,   ,   ' DJNZ,   ,   ' DJNZ,   ,   ' DJNZ,   ,   ' MOVX,   ,
( E1 ) ' AJMP,   ,   ' MOVX,   ,   ' MOVX,   ,   ' CLR,    ,   ' MOV,    ,
( E6 ) ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,
( EB ) ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,
( F0 ) ' MOVX,   ,   ' ACALL,  ,   ' MOVX,   ,   ' MOVX,   ,   ' CPL,    ,
( F5 ) ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,
( FA ) ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,   ' MOV,    ,
( FF ) ' MOV,    ,

\ Now the disassembler user interface

: EXEC-DIS  \ --
            ADDRESS C@ CELLS  OPCODE-TABLE +  @ EXECUTE
            ;

DOC  If you only want a disassembler

: DIS-WAIT?     \ -- vlag
            OPC-ADDR C@ >R              \ Save opcode to Ret-stack
            R@ $1F AND  DUP $01 =       \ Is opcode an AJMP
            SWAP $11 =  OR              \ Is opcode an ACALL
            R@ $02 =  OR                \ Is opcode a LJMP
            R@ $12 =  OR                \ Is opcode a LCALL
            R@ $22 =  OR                \ or a RET .. 
            R@ $80 =  OR                \ or a SJMP ..
            R> $73 =  OR                \ or a @A+DPTR JMP and ...
            KEY? 0=  AND                \ no key was hit ?
            IF      KEY BL <>           \ Yes wacht dan op een spatie
            ELSE    STOP?               \ Nee, behandel toetsaanslag.
            THEN
            ;

FORTH DEFINITIONS

\G Disassemble the code starting at addr in a human readable form.
\G See also: DIS
: DISASSEMBLE   ( addr -- )             \ View  
            DUP TO ADDRESS  TO OPC-ADDR  0 TO HOR  CR
            BEGIN  EXEC-DIS  CR  CLEAR HOR  DIS-WAIT? 
            UNTIL
            ;

\G Disassemble a human readable form of the code word ccc. Start
\G disassembling at the first code address and upward. This process
\G will hold when the spacebar is hit and continues when the space
\G bar will be hit again. It will quit when any other key will be hit.
: DIS       ( "ccc" -- )                \ View  Disassemble next word
            '  DISASSEMBLE
            ;

ENDDOC

ONLY  EXTRA ALSO  FORTH ALSO  DEFINITIONS

                    \ (* End of Source *) /
