\ ==============================================================
\ [TITLE]     8051-ANS-Forth decompiler words. 
\ [FILE]      8051-SEE.FRT
\ [CATEGORY]  Utilities
\ [VSN]       1.31 
\ [CREATED]   May 5, 1993, Willem Ouwerkerk 
\ [LASTCHNGD] March 22, 1996, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1993-1996 
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8051-ANS-Forth
\ [USES]      8051-ANS-Forth 
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Decompiler for 8051 source
\ ==============================================================

?def -8051-see [if]  -8051-see  [then]  MARKER -8051-see 

        (  Decompiler for 8051 Version 1.30  )

DOC  Over deze VIEW

Deze view kan bijna alle definities uit 8051-ANS-Forth
decompileren. En kan dit ook achter elkaar door doen.
Als aan het einde van een definitie moet de spatiebalk
ingedrukt wordt, dan wordt het volgende woord behandeld.

Met PWORDS worden de prefix woorden die bij een bepaald
DOxxx woord horen getoond. Maar alleen nadat een DOxxx
woord geVIEWd is. Hetzelfde geldt voor FVIEW deze toont 
de forget definitie die aan een DOxxx woord is toegekend.

VIEW is het belangrijkste woord type VIEW "name" en het
woord "name" wordt gedecompileerd.

ENDDOC

ALSO INTERNAL ALSO INSIDE  DEFINITIONS

$0002 CONSTANT JMP-CODE
$0012 CONSTANT CALL-CODE


\ Druk verschillende strings af voor de decompiler.

: .FLAGSBYTE            ( xt -- xt )
    DUP >FLAGS C@ >R  2 SPACES
    R@ $40 AND IF  ." Immediate "       THEN
    R@ $20 AND 0= IF  ." Smudged "      THEN
    R@ $04 AND IF  ." Tail "            THEN
    R> $01 AND IF  ." Compile-only "    THEN
    ;

: (.USERNAME            ( xt -- adr x )
    DUP .WORDNAME  .FLAGSBYTE           \ Druk naam en vlaggen
    CR ." Current value: "              \ Druk een tekst.
    >BODY @+ @                          \ Haal init waarde op.
    ;

: .USERNAME             ( xt -- adr )
    (.USERNAME .HEX                     \ Druk initwaarde af.
    ;

: .STRING               ( adr1 -- adr2 n )
    DUP COUNT  DUP 1+ >R  18 MIN TYPE   \ Print inline string tot max 18 tekens
    R@ +  R>                            \ Verhoog adres en <n> te dumpen bytes.
    ;


\ Vertaal en display inline gegevens ( Mag NAME? zijn ).

: STRING?   ( addr -- flag )            \ Valid Forth header string at addr ?
    COUNT DUP BL <                      \ Length may be 0 to 31
    IF      BOUNDS                      \ Check for valid ASCII characters
            ?DO     I C@ &! $7F WITHIN
            WHILE
            LOOP
                TRUE                    \ All characters valid
            ELSE
                UNLOOP  FALSE           \ Invalid character
            THEN
            EXIT
    THEN
    2DROP  FALSE                        \ Invalid count byte
    ;

: HEADER?   ( addr -- flag )            \ Check for two complete valid headers
    LOCAL ADR
    ADR CELL+ DUP STRING?               \ Check header string
    IF      COUNT + C@ $80 AND          \ Check flags byte
            IF      ADR @ ?DUP          \ Fetch link to next header
                    IF      CELL+ DUP STRING?   \ Check that header
                            IF      COUNT + C@ $80 AND \ And flags byte
                                    0>  EXIT    \ Leave flag !
                            THEN
                            DROP  FALSE  EXIT   \ Second header is invalid
                    THEN
                    TRUE  EXIT          \ End of link, all ok !
            THEN
            FALSE  EXIT                 \ Invalid flags byte !
    THEN
    DROP  FALSE  EXIT                   \ Invalid header string !
    ;

: .LITERAL              ( adr1 -- adr2 2 )
    DUP @ .HEX  2 +  2
    ;

: .CLITERAL              ( adr1 -- adr2 1 )
    DUP C@  DUP 127 >                   \ Byte-literal negatief ?
    IF  $FF00 OR  THEN                  \ Breidt het teken uit.
    .HEX  1+  1                         \ Print getal, verh. adres en
    ;                                   \ 1 byte voor de hexdump.

: .BYTEDATA              ( adr1 -- adr2 1 )
    DUP C@ .HEX  1+  1                  \ Print getal, verh. adres en
    ;                                   \ 1 byte voor de hexdump.

: .ADDRESS              ( adr1 -- adr2 2 )
    DUP @ .HEX SPACE  2 +  2            \ Druk inline getal, verh adres en
    ;                                   \ 2 bytes voor de hexdump

: .UADDRESS             ( adr1 -- adr2 1 )
    DUP @ .HEX SPACE  2 +  2            \ Druk inline getal, verh adres en *WO*
    ;                                   \ 2 bytes voor de hexdump

: .COMPILE              ( adr1 -- adr2 )
    DUP @  .WORDNAME  2 +  2            \ Druk naam van COMPILE(d) word af
    ;                                   \ 2 bytes hexdump

: .DISASS               ( adr1 -- adr2 )
    DROP  EXEC-DIS  ADDRESS      
    ;

: INLINE-DATA           ( adr1 token -- adr2 n )
    CASE                                    \ Behandel tokens met inline code.
        ['] LIT       OF  .LITERAL    ENDOF \ Literal, druk getal.
        ['] CLIT      OF  .CLITERAL   ENDOF \ Byte literal druk getal af.

        ['] (.")      OF  .STRING     ENDOF \ String, druk string af.
        ['] (S")      OF  .STRING     ENDOF \ String, druk string af.
        ['] (ABORT")  OF  .STRING     ENDOF \ Abort" , druk string af.
        ['] (COMPILE) OF  .COMPILE    ENDOF \ Druk ge(compile)erd woord af. *WO*

        ['] (TO)      OF  .UADDRESS   ENDOF \ Druk user adres af
        ['] (FROM)    OF  .UADDRESS   ENDOF \ Druk user adres af
        ['] (+TO)     OF  .UADDRESS   ENDOF \ Druk user adres af
        ['] (CLR)     OF  .UADDRESS   ENDOF \ Druk user adres af *WO*
        ['] (PUSH)    OF  .UADDRESS   ENDOF \ Druk user adres af
        ['] (POP)     OF  .UADDRESS   ENDOF \ Druk user adres af

        ['] LOCAL@    OF  .BYTEDATA   ENDOF \ Druk offset af *WO*
        ['] LOCAL!    OF  .BYTEDATA   ENDOF \ Druk offset af
        ['] LOCAL+!   OF  .BYTEDATA   ENDOF \ Druk offset af
      ['] PUSH-LOCALS OF  .BYTEDATA   ENDOF \ Druk offset af
                          0  SWAP       \ Gewoon token.
    ENDCASE
    ;

: COLON-END?    ( opcode -- vlag )
    DUP  $22 =                          \ Is het een RET
    SWAP JMP-CODE =      OR             \ een LJMP
    0=
    ;

: (TRANSLATE-CALL       ( adr1 token n1 -- adr2 n2 )
    SWAP >R  +  R>                      \ Verhoog adres1 met n1.
    DUP .WORDNAME                       \ Bepaal naam en druk deze af.
    INLINE-DATA                         \ Druk inline gegevens af.
    TAB4  ( SPACE ) .$" LCALL,"         \ Druk opcode af.
    ;                                   \ <n2> is lengte van de hexdump.

\ Kan alleen LCALL met naam correct behandelen.

: TRANSLATE-CALL        ( adr1 -- adr2 )
    DUP  .HEX  TAB1                     \ Ja, druk 8051 adres af
    DUP COMPILE@                        \ en vis dan token op
    3 (TRANSLATE-CALL  3 + .BYTES       \ Druk naam CALL en dump.
    ;

\ Kan alleen LJMP met naam en RET afhandelen

: TRANSLATE-EXIT        ( adr1 -- adr2 )
    DUP  .HEX  TAB1                     \ Ja, druk 8051 adres af
    DUP C@ $22 =                        \ Is het een RET opcode ?
    IF  ( .$" EXIT " )                  \ Ja, highlevel compiler directive.
        TAB4  .$"   RET,"  1 .BYTES     \ Druk de werkelijke opcode af.
        1+                              \ Nee, stap over RET opcode.
    ELSE
            DUP 1+ @                    \ Nee, vis na LJMP token op.
            .WORDNAME                   \ Druk naam of adres
            3 +  3                      \ Stap over code heen en <n>.
        TAB4  .$"  LJMP,"  .BYTES       \ Druk opcode en hexdump af.
    THEN
    ;

\ Kan machinecode, LCALL, LJMP en RET afhandelen.

0 VALUE NEW-DEFINITION                  \ New header found ?

: NEW-HEADER?   ( adr1 -- adr2 flag )   \ Nieuwe header gevonden ?
    DUP HEADER?                         \ Begint op adr1 een Forth header ?
    IF  LINK> DUP TO NEW-DEFINITION     \ Yes, save token
        TRUE  EXIT
    THEN
    FALSE
    ;

: TRANSLATE-TOKEN       ( adr1 -- adr2 )
    DUP TO OPC-ADDR                     \ Onthoud adres voor decompiler
    DUP TO ADDRESS                      \ Idem.
    DUP C@ COLON-END?                   \ Is het een geldig colon einde ?
    IF  DUP C@ CALL-CODE <>             \ Ok, staat er geen LCALL ?
        OVER C@ JMP-CODE <>  AND        \ en geen LJMP *WO*
        IF  
            .DISASS                     \ Ok, decompileer een opcode
        ELSE
            TRANSLATE-CALL              \ Vertaal een subroutine.
        THEN
    ELSE
        TRANSLATE-EXIT                  \ Vertaal het resultaat van EXIT
    THEN
    CR
    ;

: (DECOMPILE-ALL        ( adr -- )
    BEGIN   TRANSLATE-TOKEN  STOP?      \ Decompileer een token, vraag toets.
            >R  NEW-HEADER?  R> OR      \ Stop when a new header a found.
    UNTIL                               \ Tot einde of geen spatiebalk.
    DROP                                \ Verwijder het adres.
    100 MS
    ;

:  DECOMPILE-ALL        ( adr -- )
    ." Name : "  DUP .WORDNAME          \ Druk tekst en de naam af. 
    .FLAGSBYTE                          \ Show word data
    CR  (DECOMPILE-ALL
    ;

0 VALUE HOLD-LINK                       \ Save last private toplfa
0 VALUE HOLD-FORGET                     \ Save last forget code address

: .DOER                 ( adr -- )
    ." Doer : "  DUP .WORDNAME  .FLAGSBYTE  \ Vertaal een DOER: druk de naam af.
    CR  3 + DUP @ DUP                   \ Is er een gevuld forget field ?
    IF  ." With"                        \ Ja, meldt en bereken HOST adres.
    ELSE
        ." Without"                     \ Nee, meldt en een nul. 
    THEN
    TO HOLD-FORGET                      \ Bewaar adres.
    ."  forget code and "               \ Maak melding af.

    DUP CELL+ @                         \ Heeft deze DOER een *WO*
    @ DUP 0<>  SWAP -1 <>  AND          \ private voc ? *WO*
    IF  ." with"                        \ Ja, meldt en bereken HOST adres.
    ELSE
       ." without"                      \ Nee, meldt en een nul.
    THEN
    DUP TO HOLD-LINK                    \ sla adres op.
    ."  private voc. "                  \ maak de melding af.
    CR  3 CELLS +  (DECOMPILE-ALL       \ Decompileer de code.
    ;

: .VOC                  ( adr -- )
    ." VOCABULARY "  DUP .WORDNAME      \ Vertaal VOCABULARY
    .FLAGSBYTE  >BODY CELL+             \ Systeem info gebruikt
    @+ @ CR ." Top word: " 
    ?DUP 
    IF   LINK> .WORDNAME
    ELSE ." <Empty voc> "
    THEN CELL+
    NEW-HEADER? 2DROP CR
    ;

:  .UVAR                ( adr -- )
    ." UVARIABLE "  .USERNAME           \ Vertaal ROMVARIABLE
    NEW-HEADER? 2DROP CR
    ;

:  .UCON                ( adr -- )
    ." UCONSTANT "  .USERNAME           \ Vertaal ROMCONSTANT
    NEW-HEADER? 2DROP CR
    ;

:  .UVAL                ( adr -- )
    ." UVALUE "  .USERNAME              \ Vertaal ROMVALUE
    NEW-HEADER? 2DROP CR
    ;

:  .UVEC                ( adr -- )
    ." UVECTOR "  (.USERNAME DUP .HEX   \ Vertaal ROMVECTOR
    ." is: "   .WORDNAME                \ Druk naam routine af.
    NEW-HEADER? 2DROP CR
    ;

: .CON                  ( adr -- )
    ." CONSTANT "  DUP .WORDNAME .FLAGSBYTE \ Vertaal een CONSTANT
    CR ." Contents: "  >BODY @+ .HEX    \ Druk de inhoud af.
    NEW-HEADER? 2DROP CR
    ;

: .PREFIX               ( adr -- )
    ." PREFIX "  DUP .WORDNAME  .FLAGSBYTE  \ Vertaal een PREFIX   
    >BODY NEW-HEADER? 2DROP CR
    ;

: .FLAGS                ( adr -- )
    ." FLAG "  DUP .WORDNAME .FLAGSBYTE \ Vertaal een FLAG
    CR ." Bit address: "  >BODY C@+ .HEX \ Lees het adres
    NEW-HEADER? 2DROP CR
    ;

: .FLGS                 ( adr -- )
    ." FLG "  DUP .WORDNAME .FLAGSBYTE  \ Vertaal een FLAG
    CR ." Bit mask: " >BODY C@+ .HEX    \ Lees het adres
    NEW-HEADER? 2DROP CR
    ;

: .MARKER               ( adr -- )      \ Druk naam van marker af
    ." MARKER "  DUP .WORDNAME .FLAGSBYTE
    >BODY 34 +                          \ 'Inside' informatie
    NEW-HEADER? 2DROP CR
    ;

: .SFR                  ( adr -- )      \ Druk naam van SFR af
    ." SFR "  DUP .WORDNAME .FLAGSBYTE
    >BODY C@+ CR ." Address: "  .HEX    \ Druk SFR adres af
    NEW-HEADER? 2DROP CR
    ;

: NEXT-HEADER           ( adr1 -- adr1 adr2 )
    DUP 
    BEGIN   DUP HEADER? 0=
    WHILE   CHAR+
    REPEAT
    ;

: .CREATE               ( adr -- )
    ." CREATE "  DUP .WORDNAME .FLAGSBYTE
    >BODY CR ." Body address: "  DUP .HEX       ( body ) 
    NEXT-HEADER                                 ( body adr )
    CR ." Hexdump or Decompile, type H/D "
    KEY >CAPITAL &D <>                          ( body adr fl )
    IF   2DUP OVER - DUMP NIP                   ( adr )
         NEW-HEADER? 2DROP CR                   ( )
    ELSE DROP  CR (DECOMPILE-ALL                ( )
    THEN
    ;

: .1DATA                ( adr -- )
    DUP .WORDNAME .FLAGSBYTE
    >BODY CR ." Data: "  C@+ .HEX
    NEW-HEADER? 2DROP CR
    ;

: .BITS                 ( adr -- )
    ." BITS " .1DATA
    ;

: .MODES                ( adr -- )
    ." MODES " .1DATA
    ;

: .2DATA                ( adr -- )
    DUP .WORDNAME .FLAGSBYTE
    >BODY CR ." Data bytes: "  
    C@+ .HEX  C@+ .HEX
    NEW-HEADER? 2DROP CR
    ;

: .REG'S                ( adr -- )
    ." REG'S " .2DATA
    ;

: .SINGLE               ( adr -- )
    ." SINGLE " .2DATA
    ;

: .TABLE                ( adr -- )
    DUP .WORDNAME .FLAGSBYTE
    >BODY NEXT-HEADER
    2DUP OVER - DUMP NIP
    NEW-HEADER? 2DROP CR
    ;

: .1MODE                ( adr -- )
    ." 1MODE " .TABLE
    ;    

: .2MODES               ( adr -- )
    ." 2MODES " .TABLE
    ;    

: .RJUMP                ( adr -- )
    ." RJUMP " DUP .WORDNAME .FLAGSBYTE
    CR >BODY  @+ .WORDNAME
    NEW-HEADER? 2DROP CR
    ;

: (VIEW)                ( -- )          
    CLEAR HOLD-LINK  CLEAR HOLD-FORGET  
    CR  NEW-DEFINITION  DUP COMPILE@     \ Vis token uit LCALL/LJMP
    CASE
        ['] MODIFY     OF  .DOER   ENDOF \ Een DOER: ?
        ['] DOONLY 9 + OF  .VOC    ENDOF \ ONLY ?
        ['] DOUVOC 9 + OF  .VOC    ENDOF \ VOCABULARY ?
        ['] DOUCON 9 + OF  .UCON   ENDOF \ UCONSTANT ?
        ['] DOUVAL 9 + OF  .UVAL   ENDOF \ UVALUE ?
        ['] DOUVAR 9 + OF  .UVAR   ENDOF \ UVARIABLE ?
        ['] DOUVEC 9 + OF  .UVEC   ENDOF \ UVARIABLE ?
        ['] DOCON  9 + OF  .CON    ENDOF \ CONSTANT ?
      ['] DOPREFIX 9 + OF  .PREFIX ENDOF \ PREFIX ?
        ['] DOFLAG 9 + OF  .FLAGS  ENDOF \ FLAG ?
        ['] DOFLG  9 + OF  .FLGS   ENDOF \ FLG in flagsfield ?
      ['] DOMARKER 9 + OF  .MARKER ENDOF \ MARKER ?
        ['] DOSFR  9 + OF  .SFR    ENDOF \ SFR ?
      ['] DOCREATE 9 + OF  .CREATE ENDOF \ CREATE ?
        ['] DOBIT  9 + OF  .BITS   ENDOF \ BITS ?
        ['] DOMODE 9 + OF  .MODES  ENDOF \ MODES ?
        ['] DOREG  9 + OF  .REG'S  ENDOF \ REG'S ?
        ['] DO1    9 + OF  .SINGLE ENDOF \ SINGLE ?
        ['] DO1M   9 + OF  .1MODE  ENDOF \ 1MODE ?
        ['] DO2M   9 + OF  .2MODES ENDOF \ 2MODES ?
        ['] DORJ   9 + OF  .RJUMP  ENDOF \ RJUMP ?
        DROP  DECOMPILE-ALL    00       \ Dan colon- of code-DEFINITIE !
    ENDCASE
    ;

ALSO FORTH DEFINITIONS

\G Display a human readable form of the code starting at addr.
\G See also: VIEW
: DECOM                 ( addr -- )     \ View  Bekijk code vanaf adr.
    TO NEW-DEFINITION
    BEGIN   (VIEW)  KEY BL <> UNTIL
    ;

\G Display a human readable form of the code word ccc. Start
\G decompiling at the first code address and upward. This process
\G will hold whEn the spacebar is hit, and continues when the space
\G bar will be hit again. It wilL quit when any other key will be hit.
: VIEW                  ( ccc -- )      \ View   Bekijk Forth definities
    '  DECOM
    ;

\G Display forget routine attached to latest VIEWed word.
: FVIEW                 ( -- )          \ View   Bekijk forget code
    HOLD-FORGET ?DUP 
    IF   DECOM EXIT  
    THEN 
    ." No forget code. "
    ;

\G Show words in mini-vocabulary of latest VIEWed word.
: PWORDS                ( -- )          \ View   Laat woorden in type vocabulary zien
    HOLD-LINK CELL+ @ @ DUP 0<>  
    SWAP -1 <>  AND  
    HOLD-LINK 0<>  AND
    IF  CR  HOLD-LINK (WORDS) EXIT  
    THEN  
    ." No private voc. "
    ;

-2 SET-ORDER  DEFINITIONS               \ Zet basis zoek volgorde

                    \ (* End of Source *) /
