\ ==============================================================
\ [TITLE]     8052-ANS-Forth high level motor control words. 
\ [FILE]      MOTOR.FRT
\ [CATEGORY]  Example
\ [VSN]       1.00 
\ [CREATED]   December 3, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] August 18, 1998, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1995-1998
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 80535-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Use of compare unit and timer2 to do PWM.
\ ==============================================================
\ Parts:        Some connectors, 3 * DC-motor and a 4,5 V to 12 V
\               power supply. 3 * BUZ11, 3 * 1N4148, 3 * 47 KOhm.

\ Connect the motors to a separate 4,5 V to 12 V power supply,
\ also connect the power supply ground to the B+ ground.
\ P1.1 = CC1 = Pulsline to gate of mosfet1 (motor1)
\ P1.2 = CC2 = Pulsline to gate of mosfet2 (motor2)
\ P1.3 = CC3 = Pulsline to gate of mosfet3 (motor3)

?DEF -MOTOR [IF]  -MOTOR  [THEN]  MARKER -MOTOR

$C1 SFR CCEN                            \ Used Special Funktion Registers
$C2 SFR CCL1    $C3 SFR CCH1
$C4 SFR CCL2    $C5 SFR CCH2
$C6 SFR CCL3    $C7 SFR CCH3
$C8 SFR T2CON
$CA SFR CRCL    $CB SFR CRCH

FORTH DEFINITIONS

1000 VALUE RANGE                        \ Motor moving range.

\G Activate motor control.
: MOTOR-ON          ( -- )              \ Motor  Motor control on
    -1000      TO CRCL                  \ Set repeat time at 1 ms
    -1000 ><   TO CRCH                  \ High-byte
    0          TO CCL1                  \ Default pulsewidth1 0 ms (Low-byte)
    0          TO CCH1                  \ High-byte1
    0          TO CCL2                  \ Default pulsewidth2 0 ms
    0          TO CCH2
    0          TO CCL3                  \ Default pulsewidth3 0 ms
    0          TO CCH3
    $A8        TO CCEN                  \ Set CC1, CC2 and CC3 to compare mode
    $11        TO T2CON                 \ Start timer 2 at auto reload
    ;

\G Deactivate motor control.
: MOTOR-OFF         ( -- )              \ Motor  Motor control off
    0 TO CCEN                           \ Compare mode off
    0 TO T2CON                          \ Timer 2 off
    ;

: ADJUST            ( n1 -- n2 )        \ Keep motor speed within given range
    RANGE UMIN                          \ Keep n1 within 0 to RANGE
    NEGATE                              \ Make result negative 
    ;

\G Set motor one at speed +n which is in the range from 0 to 1000.
: MOTOR1            ( +n -- )           \ Motor  Set motor1 at speed +n
    ADJUST
    DUP >< TO CCH1
    TO CCL1
    ;

\G Set motor two at speed +n which is in the range from 0 to 1000.
: MOTOR2            ( +n -- )           \ Motor  Set motor2 at speed +n
    ADJUST
    DUP >< TO CCH2
    TO CCL2
    ;

\G Set motor three at speed +n which is in the range from 0 to 1000.
: MOTOR3            ( +n -- )           \ Motor  Set motor3 at speed +n
    ADJUST
    DUP >< TO CCH3
    TO CCL3
    ;


\ Small demo for motor one and two.

: MOTOR-DEMO        ( -- )              \ Move motor one and two
    MOTOR-ON                            \ Motor control active
    BEGIN
        RANGE 1+  0 DO  
            I MOTOR1                    \ Motor1 to speed I
            RANGE I - MOTOR2            \ Motor2 to speed RANGE - I
            400 MS  
        100 +LOOP
        -1 RANGE DO  
            I MOTOR1                    \ Motor1 to speed I
            RANGE I - MOTOR2            \ Motor2 to speed RANGE - I
            400 MS 
        -100 +LOOP
    KEY? UNTIL                          \ Until any key is hit.
    0 MOTOR1  0 MOTOR2
    ;

' MOTOR-DEMO AUTOSTART                  \ Perform motor demo on startup

\ End
