\ ==============================================================
\ [TITLE]     80535-ANS-Forth servo control words. 
\ [FILE]      SERVO535.FRT
\ [CATEGORY]  Example
\ [VSN]       1.00 
\ [CREATED]   December 1, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] August 18, 1998, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994-1998
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] ANS Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Highlevel SFR control and use of the compare unit
\ ==============================================================
\ Parts:        Some connectors, 3 servo's and a 4,5 V to 6 V power supply.

\ Connect the servo's to a separate 4,8 V to 6 V power supply.
\ Connect the power supply ground to the B+ ground.
\ P1.1 = CC1 = Pulsline servo1
\ P1.2 = CC2 = Pulsline servo2
\ P1.3 = CC3 = Pulsline servo3

?DEF -SERVO [IF]  -SERVO  [THEN]  MARKER -SERVO

HERE

$C1 SFR CCEN                            \ Used Special Funktion Registers
$C2 SFR CCL1
$C3 SFR CCH1
$C4 SFR CCL2
$C5 SFR CCH2
$C6 SFR CCL3
$C7 SFR CCH3
$C8 SFR T2CON
$CA SFR CRCL
$CB SFR CRCH

FORTH DEFINITIONS

1000 VALUE RANGE                        \ Servo moving range. Max: 1900
1000 VALUE SHORT                        \ Servo start point. Min: 800

\G Activate servo control.
: SERVO-ON          ( -- )              \ Servo  Servo control on
    -20000     TO CRCL                  \ Set repeat time at 20 ms
    -20000 ><  TO CRCH                  \ High-byte
    -1500      TO CCL1                  \ Default pulsewidth 1,5 ms (Low-byte)
    -1500 ><   TO CCH1                  \ High-byte
    -1500      TO CCL2                  \ Default pulsewidth 1,5 ms
    -1500 ><   TO CCH2
    -1500      TO CCL3                  \ Default pulsewidth 1,5 ms
    -1500 ><   TO CCH3
    $A8        TO CCEN                  \ Set CC1, CC2 and CC3 to compare mode
    $11        TO T2CON                 \ Start timer 2 at auto reload
    ;

\G Deactivate servo control.
: SERVO-OFF         ( -- )              \ Servo  Servo control off
    0 TO CCEN                           \ Compare mode off
    0 TO T2CON                          \ Timer 2 off
    ;

: ADJUST            ( n1 -- n2 )        \ Keep servo within given range
    RANGE UMIN                          \ Keep n1 within 0 to RANGE
    SHORT +                             \ Add SHORTest pulse to it
    NEGATE                              \ Make result negative 
    ;

\G Set servo one in position +n which is in the range from 0 to 1000.
: SERVO1            ( +n -- )           \ Servo  Set servo1 at position +n
    ADJUST
    DUP >< TO CCH1
    TO CCL1
    ;

\G Set servo two in position +n which is in the range from 0 to 1000.
: SERVO2            ( +n -- )           \ Servo  Set servo2 at position +n
    ADJUST
    DUP >< TO CCH2
    TO CCL2
    ;

\G Set servo three in position +n which is in the range from 0 to 1000.
: SERVO3            ( +n -- )           \ Servo  Set servo3 at position +n
    ADJUST
    DUP >< TO CCH3
    TO CCL3
    ;


\ Small demo for servo one and two.

: SERVO-DEMO        ( -- )              \ Move servo one and two
    SERVO-ON                            \ Servo control active
    BEGIN
        1001  0 DO  
            I SERVO1                    \ Servo1 to position I
            1000 I - SERVO2             \ Servo2 to position 1000 - I
            200 MS  
        100 +LOOP
        -1 1000 DO  
            I SERVO1                    \ Servo1 to position I
            1000 I - SERVO2             \ Servo2 to position 1000 - I
            200 MS 
        -100 +LOOP
    KEY? UNTIL                          \ Until any key is hit.
    ;

HERE SWAP - .

' SERVO-DEMO AUTOSTART                  \ Perform servo demo on startup

\ End
