\ Test code to switch clockspeed for Dallas 80C320 etc.

?DEF -TEST  [IF]  -TEST  [THEN]  MARKER -TEST

$8E CONSTANT CKCON:
$D8 CONSTANT WDCON:
$C7 CONSTANT TA:
$E8 CONSTANT EIE:
$A8 CONSTANT IE:
$99 CONSTANT SBUF:

$87 CONSTANT PCON:
$91 CONSTANT EXIF:

\ Part-1: Use of clock stretch for MOVX !!

\G Change the duration of a MOVX instruction from 2 to 9 
\G machine cycles. If +n is 0 a MOVX takes 2 cycles, when +n
\G is 1 then MOVX takes 3 cycles etc. The value of +n is 
\G maximal 7, then MOVX takes 9 cycles.
CODE STRETCH        ( +n -- )           \ EXTRA  Stretch external R/W clock
    R0: INC,
    A: @R0 MOVX,
    R0: INC,
    A: 7 # ANL,
    CKCON: A: ORL,
    RET,
END-CODE

\ Part-2: Use of the stop mode !

CODE STOP           ( -- )              \ EXTRA  Processor in STOP mode
    PCON: $02 # ORL,
    RET,
END-CODE

CODE IDLE           ( -- )              \ EXTRA  Processor in IDLE mode
    PCON: $01 # ORL,
    RET,
END-CODE

CODE RING           ( -- )              \ EXTRA  Ring oscilator on
    EXIF: $02 # ORL,
    RET,
END-CODE

\ Part-3: Use of watchdog timer !

\ A reset by the watchdog hangs the Forth system !!
\ What must we do to make this work ?????

\G Set watchdog timer on with a timeout of 2-17, 2-20, 2-23
\G or 2-26 crystal cycles ( + 512 ).
CODE WATCHDOG-ON    ( x -- )            \ EXTRA  Set watchdog timer on
    R0: INC,
    A: @R0 MOVX,
    R0: INC,
    A: 3 # ANL,
    A: RR,
    A: RR,
    CKCON: $3F # ANL,                   \ Clear timeout bits in CKCON
    CKCON: A: ORL,                      \ Pack CKCON with new timeout bits

    TA: $AA # MOV,                      \ Timed access
    TA: $55 # MOV,
    WDCON: 3 # MOV,                     \ Watchdog timer and reset on
\   WDCON: 1 # MOV,                     \ Watchdog timer on
    EIE: .4 SETB,                       \ Enable watchdog interrupt
    IE: .7 SETB,                        \ Enable all interrupts
    RET,
END-CODE

CODE SETUP-WATCHDOG ( -- )              \ Satisfy 80520 interrupt hardware
    RETI,
END-CODE

\G Set the watchdog timer off
CODE WATCHDOG-OFF   ( -- )              \ EXTRA  Set watchdog timer off
    TA: $AA # MOV,                      \ Timed access
    TA: $55 # MOV,
    WDCON: $F0 # ANL,                   \ Watchdog timer off
    EIE: .4 CLR,                        \ Disable watchdog interrupt
    RET,
END-CODE

\G Reset the watchdog timer
CODE RESET-WATCHDOG ( -- )              \ EXTRA  Reset watchdog timer
    TA: $AA # MOV,                      \ Timed access
    TA: $55 # MOV,
    WDCON: .0 SETB,                     \ Restart watchdog timer
    RET,
END-CODE

\G Has a watchdog reset occured, leave true on the stack, otherwise
\G false, then clear the watchdog-reset bit.
CODE WATCHDOG?      ( -- flag )         \ EXTRA  Watchdog timeout occurred
    C: WDCON: .2 MOV,                   \ Read the WTRF flag
    TA: $AA # MOV,                      \ Timed access
    TA: $55 # MOV,
    WDCON: .2 CLR,                      \ Clear the WTRF flag

\   C: WDCON: .3 MOV,                   \ Read the WDIF flag
\   TA: $AA # MOV,                      \ Timed access
\   TA: $55 # MOV,
\   WDCON: .3 CLR,                      \ Clear the WDIF flag
\   C: EIE: .4 ANL,                     \ Used to detect a normal reset

    A: ACC: SUBB,
    R0: DEC,
    @R0 A: MOVX,
    R0: DEC,
    @R0 A: MOVX,
    RET,
END-CODE

\ Part-4: Power fail interrupt

\ Power fail is almost useless, because the external RAM is
\ switched off at the same time. The micro can't execute then.

CODE POWER-FAIL-ON  ( -- )              \ Enable power fail interrupt
    WDCON: .4 CLR,                      \ Clear interrupt
    WDCON: .5 SETB,                     \ Enable new interrupts
    RET,
END-CODE

CODE POWER-FAIL-OFF ( -- )              \ Enable power fail interrupt
    WDCON: .4 CLR,                      \ Clear interrupt
    WDCON: .5 CLR,                      \ Disable new interrupts
    RET,
END-CODE


\ Some demo routines

: POWER-FAIL        ( -- )
    SETUP-WATCHDOG
    POWER-FAIL-ON
    CR ." Warning power is failing ! "
    ;

: WATCHDOG          ( x -- )
    WATCHDOG-ON 
    BEGIN
        200 MS  ." ."  
        200 MS  ." ."  
\       WATCHDOG?
\       IF  CR ." Watchdog activated "
\       THEN
        RESET-WATCHDOG 
    KEY? UNTIL
    WATCHDOG-OFF
    ;

: WATCH             ( -- )              \ Show active watchdog
    WATCHDOG?
    IF      CR ." Watchdog activated "
            7 EMIT
    THEN    
    2 WATCHDOG-ON
    ;

CODE WAIT           ( -- )
    R2: 0 # MOV,
    DO,     NOP,
            NOP,
            NOP,
    R2: LOOP,
    RET,
END-CODE

$40 SFR TEL

CODE PEST           ( -- )
    ADR TEL INC,
    A: ADR TEL MOV,
    ZER IF,
        SBUF: &F # MOV,                     \ Type string
        ' WAIT CALL,
        SBUF: &r # MOV,
        ' WAIT CALL,
        SBUF: &o # MOV,
        ' WAIT CALL,
        SBUF: &t # MOV,
        ' WAIT CALL,
        SBUF: &h # MOV,
        ' WAIT CALL,
        SBUF: BL # MOV,
        ' WAIT CALL,
    THEN,
    TA: $AA # MOV,                      \ Timed access
    TA: $55 # MOV,
    WDCON: .3 CLR,                      \ Clear the WDIF flag
    RETI,
END-CODE

: WO    WATCHDOG-OFF ;

' PEST          WDTI-VEC  SET-VECTOR    \ Install watchdog interrupt
\ ' POWER-FAIL    PFI-VEC   SET-VECTOR    \ install power fail interrupt
160 TO FUDGE                            \ Set MS trim variable

' WATCH     AUTOSTART                   \ Restart test of watchdog

\ End
