\ ==============================================================
\ [TITLE]     8051 ANS-Forth ACCEPT word set. 
\ [FILE]      ACCEPT.FRT
\ [CATEGORY]  Example
\ [VSN]       1.00 
\ [CREATED]   June 21, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] October 18, 1994, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Example of how to change a system vector and so
\             change the systems behaviour. And the use of a
\             high level bit flag. (Do not use with ETX/ACK)
\ ==============================================================

MARKER -ACCEPT

\ A small accept routine as source for 8051-ANS-Forth

INTERNAL ALSO DEFINITIONS

         0 VALUE CNT                    \ Current chracter pointer
         0 VALUE PCNT                   \ Last character in prev. line
         0 VALUE MAXCNT                 \ Maximum number of accepted chars.
         0 VALUE BUFF                   \ Address of current input buffer

FORTH DEFINITIONS

\G The returned flag is true when SACCEPT is left using the ESC
\G key, otherwise flag is false.
         FLAG ESC?  ( -- flag )         \ Accept 
           
\ Special output codes

INTERNAL DEFINITIONS

       $07 CONSTANT =BELL               \ Bell ASCII code
       
\ Special keys on the keyboard

       $08 CONSTANT =BS                 \ Backspace ASCII code
       $1B CONSTANT =ESC                \ Escape ASCII code

: (BS)      ( -- )                      \ Perform destructive a backspace
    BS  SPACE  BS
    ;

: BELL      ( -- )                      \ Ring terminal bell
    =BELL EMIT
    ;

: DO-BS     ( -- )                      \ Perform BS when possible
    CNT  
    IF   -1 +TO CNT  (BS)  EXIT 
    THEN
    BELL  
    ;

: DO-DEL    ( -- )                      \ Delete whole input line
    CNT 0 ?DO  (BS)  LOOP  
    CLEAR CNT
    ;

: DO-CR     ( -- u true )               \ Accept current input line
    CNT  DUP TO PCNT  SPACE  TRUE
    ;

: DO-ESC    ( -- 0 true )               \ Escape from current input line
    DO-DEL  SET ESC?  SPACE  0  TRUE
    ;

: DO-REST   ( -- false )                \ Restore input line from buffer
    BEGIN   CNT PCNT <>
    WHILE   BUFF CNT + C@ EMIT
            1 +TO CNT
    REPEAT  FALSE
    ;

: HANDLE-CHAR   ( char -- false )       \ Store character in input buffer
    MAXCNT CNT < 0=
    IF   DUP BUFF CNT +  C!
         EMIT  1 +TO CNT
    ELSE DROP BELL
    THEN FALSE
    ;

DOC  SACCEPT

'SACCEPT' has the following funktions:

Escape key       = Escape SACCEPT
^E               = Erase full text line
^R               = Restore text line 
Backspace or     = Delete last character

ENDDOC

FORTH DEFINITIONS

\G Receive a string of at most +n1 characters. Bits representing
\G higher numbers as 32767 are ignored. Input terminates when
\G the escape or enter key is pressed. +n2
\G  Escape key   = Escape from SACCEPT +n2 is zero and ESC? is true.
\G  ^E           = Erase full text line
\G  ^R           = Restore text line 
\G  Backspace or = Delete last character
\G  Enter        = Accept current input line +n2 are the number of
\G                 characters received at the input line and ESC?
\G                 is false.
: SACCEPT   ( c-addr +n1 -- +n2 )       \ Accept 
    $7FFF AND TO MAXCNT  TO BUFF  
    CLEAR CNT  CLEAR ESC?
    BEGIN
        KEY  DUP BL <
        IF
            CASE
                =BS   OF  DO-BS  FALSE     ENDOF
                ^R    OF  DO-REST          ENDOF
                ^E    OF  DO-DEL FALSE     ENDOF
                ^M    OF  DO-CR            ENDOF
                =ESC  OF  DO-ESC           ENDOF
                          HANDLE-CHAR DUP
            ENDCASE
        ELSE
            HANDLE-CHAR
        THEN
    UNTIL 
    ;

\G Install original ACCEPT routine
: OLD-ACCEPT    ( -- )                  \ Accept
    [ ADR ACCEPT @ ] LITERAL  IS ACCEPT 
    ;

\G Install SACCEPT as ACCEPT routine
: NEW-ACCEPT    ( -- )                  \ Accept
    ['] SACCEPT               IS ACCEPT 
    ;

ONLY  EXTRA ALSO  FORTH ALSO  DEFINITIONS

                    \ (* End of Source *) /
