\ ==============================================================
\ [TITLE]     8051 ANS Forth BAMBOE word set. 
\ [FILE]      BAMBOE.FRT
\ [CATEGORY]  Example
\ [VSN]       1.01
\ [CREATED]   January 16, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] August 19, 1998, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994-1998
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   16 bits I/O, Bamboe using 5 io-port bits on port 1
\             Example of use of machine code compared to high level
\             and the use of METHODS and DOER:
\ ==============================================================
\ 27-11-95    Added delays for Theo v. Lottum's BAMBOE board.

MARKER -BAMBOE  CR .( Test BAMBOE routines from cursus sheets )

ONLY  FORTH ALSO  INTERNAL ALSO  EXTRA ALSO
INTERNAL DEFINITIONS

TRUE  [IF]

$10 CONSTANT #BITS

\ Connections to port P1
\
\ Bit 0 = Free
\ Bit 1 = Free
\ Bit 2 = Free ... but connected to 10K grounded resistor and a gate.
\ Bit 3 = Load input
\ Bit 4 = Output ready
\ Bit 5 = Clock
\ Bit 6 = Data to Bamboe outputs
\ Bit 7 = Data from Bamboe inputs

CODE BAMBOE!  ( x -- )                 \ At 12 MHz it takes 159 microsec.
    A: @R0 MOVX,                       \ Get highbyte
    R3: A: MOV,                        \ Save it in R3
    R0: INC,                           \ Pop highbyte
    A: @R0 MOVX,                       \ Get lowbyte, keep in accu
    R0: INC,                           \ Pop lowbyte
    P1: .3 SETB,                       \ Init. (don't sample input bits).
    R2: #BITS # MOV,                   \ A Bamboe has 16 bits
    DO,
        A: RLC,                        \ Shift to left thru carry
        A: R3: XCH,                    \ Get highbyte
        A: RLC,                        \ Shift to left including carry
        A: R3: XCH,                    \ Put it back, get lowbyte

        P1: .6 C: MOV,                 \ Set one bit ready
        P1: .5 CLR,                    \ Send data, make clockbit low 
        P1: .5 SETB,                   \ Set data, make clockbit high
    R2: LOOP,                          \ Do this 16 times
    P1: .4 SETB,                       \ Ready, shiftreg. to outputs
    P1: .4 CLR,                        \ And disconnect them again 
    RET,                               \ End
END-CODE

CODE BAMBOE@    ( -- x )               \ At 12 MHz it takes 161 microsec.
    P1: .3 CLR,                        \ Load sample inputs to shiftregister
    P1: .7 SETB,                       \ Init. (inputbit activated).
    NOP, NOP, NOP,                     \ Extra delay for some cables
    P1: .3 SETB,                       \ Disconnect inputs.
    NOP, NOP, NOP,
    R2: #BITS # MOV,                   \ Bamboe has 16 bits
    DO,
        C: P1: .7 MOV,                 \ Read one data bit
        P1: .5 CLR,                    \ Get new databit, make clockbit low
        P1: .5 SETB,                   \ Make clockbit high
        NOP, NOP, NOP,                 \ Extra delay for some cables

        A: RRC,                        \ Shift received bit in it to right
        A: R3: XCH,                    \ Exchange with low byte
        A: RRC,                        \ Shift overflow and content to right
        A: R3: XCH,                    \ Exchange with high byte
    R2: LOOP,                          \ Do this 16 times
    R0: DEC,                           \ Make stackspace
    A: CPL,                            \ Complement all bits
    @R0 A: MOVX,                       \ Set at lowbyte position
    R0: DEC,                           \ Make stackspace
    A: R3: MOV,                        \ Get lowbyte
    A: CPL,                            \ Complement all bits
    @R0 A: MOVX,                       \ Set at highbyte position
    RET,                               \ End
END-CODE

[ELSE]

ASSEMBLER
P1: SFR POORT                         \ Port 1 of 80C535, 80C32, etc.
FORTH

$00 CONSTANT LEES-INGANG              \ Bit patterns for wanted functions
$18 CONSTANT UITGANG-KLAAR 
$80 CONSTANT DATA-IN
$40 CONSTANT DATA-UIT
$28 CONSTANT KLOK
$08 CONSTANT NUL

$10 CONSTANT #BITS                     \ Number of output bits
 
: SCHRIJF-BIT  ( x -- )                \ Write one bit
    DUP TO POORT  KLOK OR TO POORT 
    
 
: BAMBOE!      ( x -- )                \ Write #BITS bits
    0 #BITS 1- 
    DO
        DUP  1 I LSHIFT  AND 0<>
        DATA-UIT AND  NUL OR  SCHRIJF-BIT
    -1 +LOOP
    DROP  UITGANG-KLAAR TO POORT 
    ;
 
: LEES         ( -- x )                \ Read input port
    DATA-IN NUL OR TO POORT  POORT 
    ;

: LEES-BIT     ( -- x )                \ Leave $8000 when input is zero
    LEES DATA-IN AND 0=   $8000 AND 
    ;
 
: BAMBOE@      ( -- x )                \ Read #BITS bamboe input bits
    LEES-INGANG TO POORT  NUL TO POORT
    0 #BITS 0
    DO  1 RSHIFT 
        LEES-BIT  OR
        NUL TO POORT  KLOK TO POORT
    LOOP 
    ><
    ;

[THEN]


DOER: DOBAMBOE                          \ Build a DOER for BAMBOE
    DROP  FLYER  POSTPONE BAMBOE@ 
    ;

METHODS DOBAMBOE                        \ Add TO to its prefix operators

: TO    DROP  POSTPONE BAMBOE! ;

END-METHODS

FORTH DEFINITIONS  INTERNAL

\G Leave the status of the BAMBOE inputs on the stack. When TO is
\G used as a prefix ( TO BAMBOE ) the BAMBOE outputs will be changed.
CREATE BAMBOE  IMMEDIATE  DOBAMBOE ( -- x ) \ Bamboe

ONLY  EXTRA ALSO  FORTH ALSO

                    \ (* End of Source *) /
