\ ==============================================================
\ [TITLE]     8051 ANS Forth BLOCK words set. 
\ [FILE]      BLOCK.FRT
\ [CATEGORY]  Example
\ [VSN]       1.00 
\ [CREATED]   April 9, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] October 20, 1994, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Show how blocks can be added to 8051-ANS-Forth
\             In this example screen 0 does not exists !!
\             This is an incomplete implementation, just for
\             demonstration and testing purposes.
\ ==============================================================

MARKER -BLOCKS

\G Transform the search order so that the EDITOR vocabulary is
\G searched first.
VOCABULARY EDITOR   ( -- )              \ Blocks 

\G If u1 is greater than zero, u2 is equal to u1 minus the number
\G of spaces at the end of the character string specified by
\G c-addr u1. If u1  is zero or the entire string consists of
\G spaces, u2 is zero.
: -TRAILING ( c-addr u1 -- c-addr u2 )  \ Blocks  Remove trailing spaces
    DUP 0
    ?DO     2DUP + CHAR- C@ BL <>
            IF      LEAVE
            THEN
            CHAR-
    LOOP
    ;

INTERNAL ALSO DEFINITIONS

0 VALUE LINE                            \ Hold current line number
0 VALUE BLKS                            \ Hold number of block buffers

FORTH DEFINITIONS

\G a-addr is the address of a cell containing the block number of
\G the block most recently LISTed.
VARIABLE SCR   ( -- a-addr )            \ Blocks  Current active screen

INTERNAL DEFINITIONS

$400 CONSTANT BLOCK-SIZE                \ Length of a block buffer
 $40 CONSTANT LINE-LENGTH               \ Length of one screen line

FORTH DEFINITIONS

\G Reserve +n Block buffers, if +n is negative or larger than
\G 20 it will be replaced by 20. Take care 20 block buffers
\G will take 20 Kbyte of dictionary space.
: BLOCKS    ( +n -- )                   \ Blocks  Reserve <+n> block buffers
    20 UMIN
    DUP NEGATE BLKS +
    BLOCK-SIZE *
    +TO BARRIER
    TO BLKS
    ;

INTERNAL DEFINITIONS

: (BLOCK)   ( u -- adr )                \ Blocks  Give address of current buffer
    DUP BLKS 1+ 1 WITHIN  
    BLKS 0=  OR
    ABORT" Invalid block number"
    1- BLOCK-SIZE *
    BARRIER +
    ;

' (BLOCK) TO BLOCK                      \ Blocks  Set the BLOCK vector

EDITOR ALSO DEFINITIONS

\G Fill screen u with blanks
: WIPE      ( u -- )                    \ Blocks  Fill screen <u> with blanks
    DUP TO SCR
    BLOCK BLOCK-SIZE BL FILL
    ;

FORTH DEFINITIONS

\G Display block u at the current output device. Store u in SCR .
: LIST      ( u -- )                    \ Blocks  Type screen <u>
    DUP TO SCR  
    CR ." SCR # "  DUP .
    BLOCK
    16  0
    DO      CR I 2 .R SPACE
            DUP I LINE-LENGTH * +
            LINE-LENGTH -TRAILING TYPE
    LOOP
    DROP
    ;

\G Save the current input-source specification. Store u in BLK .
\G (thus making block u the input source), set >IN to zero and
\G interpret. When the parse area is exhausted, restore the prior
\G input source specification. Other stack effects are due to the
\G word loaded. If u is zero issue an error message.
: LOAD      ( i*x u -- j*x )            \ Blocks  Interpret screen <u>
    PUSH BLK   PUSH >IN
    TO BLK  CLEAR >IN
    INTERPRET
    POP >IN  POP BLK
    ;

\ A very small editor for those ram blocks

EDITOR DEFINITIONS

\G Display the current screen on the current output device.
\G The variable SCR holds the current screen.
: L         ( -- )                      \ Blocks  List current screen
    PAGE  FROM SCR LIST
    ;

\G Make next screen the current screen, increasing SCR with one.
\G The screen number SCR will never be greater than the maximum
\G number of screens reserved. Perform the function of L .
: N         ( -- )                      \ Blocks  List next screen
    FROM SCR  1 BLKS WITHIN
    IF      1 +TO SCR
    ELSE    BLKS TO SCR
    THEN
    L
    ;

\G Make previous screen the current screen, increasing SCR with
\G one. The screen number SCR will never be smaller than 1.
\G Perform the function of L .
: B         ( -- )                      \ Blocks  List previous screen
    FROM SCR  2 BLKS 1+ WITHIN
    IF      -1 +TO SCR
    ELSE    1 TO SCR
    THEN
    L
    ;

INTERNAL DEFINITIONS

: (LINE)    ( +n -- adr )               \ Blocks  Get line address within a block
    DUP 16 0 WITHIN ABORT" Invalid line number"
    TO LINE  
    FROM SCR BLOCK  
    LINE LINE-LENGTH *  +
    ;

EDITOR DEFINITIONS

\G Put the string represented by ccc at line +n of the current
\G screen, thus overwriting the previous contents. Perform the
\G function of L after altering the screen.
: P         ( +n "ccc" -- )             \ Blocks  Put <ccc> at line <+n>
    (LINE)
    13 PARSE-WORD
    CR 2DUP TYPE
    ROT SWAP MOVE
    L
    ;

INTERNAL DEFINITIONS

: (E)       ( +n -- )                   \ Blocks  Erase line <+n> primitive
    (LINE) LINE-LENGTH BL FILL
    ;

EDITOR DEFINITIONS

\G Delete line +n of the current screen, the previous contents
\G of that line are lost. Perform the function of L after altering
\G the screen.
: E         ( +n -- )                   \ Blocks  Erase line <+n>
    (E)  L
    ;

\G Replace line +n with the string ccc. Perform the function of L
\G after altering the screen.
: R         ( +n "ccc" -- )             \ Blocks  Replace line <+n> with <ccc>
    DUP (E)  P
    ;

\G Insert the string ccc at line +n the last line of the screen
\G will be lost. Perform the function of L after altering the
\G screen.
: I         ( +n "ccc" -- )             \ Blocks  Insert <ccc> at line <+n>
    >R  R@ (LINE)
    DUP LINE-LENGTH +
    15 LINE - 
    LINE-LENGTH *  MOVE
    R@ (E)
    R> P
    ;

\G Cut line +n from the current screen, all lines below line +n
\G move one line upwards. Line 15 is filled with blanks. Perform
\G the function of L after altering the screen.
: D         ( +n -- )                   \ Blocks  Delete line <n> shift all others
    (LINE)  DUP LINE-LENGTH +  SWAP
    15 LINE -
    LINE-LENGTH *  MOVE
    15 E
    ;

PREVIOUS PREVIOUS
FORTH DEFINITIONS

                    \ (* End of Source *) /
