\ ==============================================================
\ [TITLE]     8052-ANS-Forth duty cycle with timer 1 test. 
\ [FILE]      DUTY.FRT
\ [CATEGORY]  Test program
\ [VSN]       1.00 
\ [CREATED]   March 8, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] August 19, 1998, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1995-1998
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8052-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Using timer 1 to create Pulse Width at P1.0
\             For use with DIMMER program on AT89C2051
\
\     Pulse in  0---[###]---,------0  Analog out
\                 100K      |
\                         =====
\                           | 1u
\                           |
\ ==============================================================

?DEF -DUTY [IF]  -DUTY  [THEN]  MARKER -DUTY

ASSEMBLER ALSO  DEFINITIONS

: =:    CONSTANT ;

    $89 =: TMOD:                        \ 80C535 assembler labels
    $8D =: TH1:
    $8B =: TL1:
    $AB =: ET1:
    $8E =: TR1:
    $AF =: EAL:

$34 =: HIGH-VALUE
$35 =: LOW-VALUE

$35 =: PULSE?

FORTH DEFINITIONS  ASSEMBLER

LABEL DUTY-CYCLE                        \ 12 Cycles en 40 interupts/sec
    PULSE? BS IF,
        TH1: HIGH-VALUE MOV,
        TL1: $FF # MOV,
        P1: .0 SETB,
        PULSE? CLR,
        RETI,
    THEN,
    TH1: LOW-VALUE MOV,
    TL1: $FF # MOV,
    P1: .0 CLR,
    PULSE? SETB,
    RETI,
END-CODE

\G Set pulswidth +n of PWM ouput signal, +n is from 0 to 100.
CODE SET-VALUE  ( +n -- )               \ Duty
    R0: INC,                            \ Get +n from stack
    A: @R0 MOVX,
    R0: INC,

    R7: A: MOV,                         \ Keep within 0 and 99
    C: CLR,
    A: 100 # SUBB,
    CS IF,      A: R7: MOV,
    ELSE,       A: 99 # MOV,
    THEN,

    R7: A: MOV,                         \ Negate high part and save
    A: CPL,
    HIGH-VALUE A: MOV,

    A: 99 # MOV,                        \ Calculate low part, negate
    C: CLR,                             \ and save
    A: R7: SUBB,
    A: CPL,
    LOW-VALUE A: MOV,
    RET,
END-CODE

\G Activate duty cycle at pin 1.0
CODE ON-DUTY    ( -- )                  \ Duty
    ] 0 SET-VALUE  DUTY-CYCLE TF1-VEC SET-VECTOR [
    TMOD: $0F # ANL,                    \ Clear Timer 1 mode bits
    TMOD: $10 # ORL,                    \ Set Timer 1 mode 1
    TH1: $FF # MOV,                     \ Set negated 16 bits count
    TL1: $FF # MOV,
    ET1: SETB,                          \ Enable timer 1 interrupts
    TR1: SETB,                          \ Start timer 1
    EAL: SETB,                          \ Global enable interupts
    RET,
END-CODE

\G Deactivate duty cycle at pin 1.0
CODE OFF-DUTY   ( -- )                  \ Duty
    EAL: CLR,                           \ Global disable interupts
    ET1: CLR,                           \ Disable timer 0 interrupts
    TR1: CLR,                           \ Stop timer 0
    ] TF1-VEC CLEAR-VECTOR [
    RET,
END-CODE

: DEMO          ( -- )                  \ Show changing PWM signal
    ON-DUTY
    BEGIN
        100 0 
        DO  
            I SET-VALUE  50 MS          \ Set new pulswidth and wait
            STOP? IF  LEAVE  THEN
        LOOP
        STOP?
    UNTIL
    OFF-DUTY
    ;

-2 SET-ORDER  DEFINITIONS

                      \ (* End of Source *) /
