\ ==============================================================
\ [TITLE]     8051 ANS Forth EBAMBOE word set. 
\ [FILE]      EBAMBOE.FRT
\ [CATEGORY]  Example
\ [VSN]       1.00 
\ [CREATED]   January 16, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] October 19, 1994, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      BAMBOE.FRT
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Examples of use of BAMBOE input and output
\ ==============================================================

ROM

ONLY  FORTH ALSO  INTERNAL ALSO  EXTRA ALSO

MARKER -VBAMBOE

\ Times .....

INTERNAL DEFINITIONS

1 VALUE #times                          \ TIMES counter

FORTH DEFINITIONS

\ Execute the command line u times
: TIMES     ( u -- )                    \ Ebamboe 
        1 +TO #times  #times U<         \ Increase and compare counter.
        IF      1 TO #times  EXIT       \ Equal to value from stack? ready!
        THEN    CLEAR >IN               \ Execute command line again.
        ;

\ Set the numeric conversion radix to two (binary).
: BINARY    #2 TO BASE ;                \ Ebamboe  Set base to #2

\ Print a single word x from the stack in binary. All 16 bits
\ will be printed. The current number base is not changed.
: .BIN      ( x -- )                    \ Ebamboe 
        PUSH BASE  BINARY               \ Save and change BASE
        0 <#  16 0 DO # LOOP  #>        \ Convert 16 digits
        TYPE SPACE                      \ Print string
        POP BASE                        \ Restore BASE
        ;

\G Wait from 0 to 255 milleseconds depending on the low byte of
\G the input bits of the BAMBOE .
: WAIT      ( -- )                      \ Ebamboe 
        BAMBOE $FF AND  MS              \ Wait maximal 255 ms
        ;

\ Leave true on the stack when a character is available.
: QUIT?     ( -- flag )                 \ Ebamboe 
        KEY?                            \ Stop on any key.
        ;

\ Some examples

\G Display all BAMBOE inputs, directly at its outputs.
: I/O       ( -- )                      \ Ebamboe 
        BEGIN   BAMBOE                  \ Read BAMBOE inputs
                TO BAMBOE               \ Write back to outputs
                QUIT?                   \ Do so until a key is hit
        UNTIL
        ;

\G Simulate a walking light at the BAMBOE outputs, the speed may
\G be adjusted by the low byte of the BAMBOE inputs.
: WALKING-LIGHT ( -- )                  \ Ebamboe 
        0
        BEGIN   DUP TO BAMBOE           \ Send bit pattern to outputs
                2*                      \ Shift to left
                $FFFF AND  ?DUP 0=      \ Did we handle 16 bits
                IF      1               \ Start with begin value again
                THEN
                WAIT                    \ Wait low-byte times MS
                QUIT?                   \ Do so until a key is hit
        UNTIL
        DROP                            \ Remove bit pattern
        ;

\G Simulate a special kind of walking light at the BAMBOE outputs,
\G the speed may be adjusted by the low byte of the BAMBOE inputs.
: KNIGHT-RIDER  ( -- )                  \ Ebamboe 
        BEGIN   1  $10  0               \ Start value and loop bounds
                DO      DUP TO BAMBOE   \ Send bit pattern to BAMBOE
                        WAIT            \ Wait low-byte times MS
                        1 LSHIFT        \ Shift data to left
                        QUIT? IF LEAVE THEN \ Do until a key is pressed
                LOOP                    \ Until all 16 bits are done
                DROP  2  $4000          \ Remove data and setup second loop
                DO      I TO BAMBOE     \ Send bit pattern to BAMBOE outputs
                        WAIT            \ Wait low-byte times MS
                        QUIT? IF LEAVE THEN \ Do until a key is pressed
                I 2/ NEGATE +LOOP       \ Decrease loop index
                QUIT?                   \ Do until any key is pressed
        UNTIL
        ;

ONLY  EXTRA ALSO  FORTH ALSO  DEFINITIONS

                    \ (* End of Source *) /
