\ ==============================================================
\ [TITLE]     8051 ANS Forth 13 keys KEYBOARD words set. 
\ [FILE]      KEYBOARD.FRT
\ [CATEGORY]  Example
\ [VSN]       1.00 
\ [CREATED]   April 19, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] April 19, 1994, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Reading a key from a 13-keys dump telephone keyboard
\             Connected in a row column matrix. A keyboard like
\             this can be bought for example for 3,95 at INTRONICS
\ ==============================================================

\G            Connected in a row column matrix, like this:
\G
\G            Keyboard at port-1
\G
\G            bit of port-1 0 - 1 - 2 - 3
\G                          |   |   |   |
\G                     4----1---4---7---* 
\G                          |   |   |   |
\G                     5----2---5---8---0
\G                          |   |   |   |
\G                     6----3---6---9---#
\G
\G            How are the bits used:
\G
\G            Bit 0 = Row with 1 2 3 keys
\G            Bit 1 = Row with 4 5 6 keys
\G            Bit 2 = Row with 7 8 9 keys
\G            Bit 3 = Row with * 0 # keys
\G            Bit 4 = Column with 1 4 7 * keys
\G            Bit 5 = Column with 2 5 8 0 keys
\G            Bit 6 = Column with 3 6 9 # keys
\G            Bit 7 = R key

ROM

INTERNAL ALSO  DEFINITIONS

ASSEMBLER
P1: SFR P1                              \ Access port 1
FORTH

\ Leave true if the R key is pressed on the keyboard
: RKEY          ( -- Flag )             \ Keyboard 'R' key pushed ?
    P1 $80 AND 0=
    ;

\ Get column number of the pressed key, leave -1 if no key
\ is pressed.
: COLUMN        ( -- +n|-1 )            \ Keyboard  Get column of pushed key
    P1 INVERT $F AND
    4  0
    DO      DUP  1 I LSHIFT  =
            IF      DROP  I  
                    UNLOOP EXIT
            THEN
    LOOP    DROP  -1
    ;

\ Get row and column of the pressed key.
: SCAN-KEYB     ( -- Row Column )       \ Keyboard  Get row column of pushed key
    3  0
    DO      $10 I LSHIFT  INVERT TO P1
            P1 $F AND $F <
            IF      I 1+  COLUMN
                    UNLOOP  EXIT
            THEN
    LOOP    -1  -1
    ;

\ Get the key's scan code on the stack, leave -1 if no key is
\ pressed include the R-key.
: SCAN-KEY      ( -- 0to12|-1)          \ Keyboard  Get scan code of a key
    SCAN-KEYB S>D
    IF      2DROP  RKEY
            IF      0   EXIT
            THEN    -1  EXIT
    THEN    3 *  +
    ;

\ Hold scan code of latest pressed key.
0 VALUE KEY-NR                          \ Keyboard  Hold scan code

FORTH DEFINITIONS

\G Leave true if any key is pressed, leave the scan code in
\G the value KEY-NR .
: AKEY?         ( -- Flag )             \ Keyboard 
    SCAN-KEY  DUP TO KEY-NR  0< 0=      \ Any key pushed ?
    ;

INTERNAL DEFINITIONS

\ Define a translation table, so that the ASCII values printed
\ on the keys correspond to the value returned by AKEY .
CREATE KEY-TABLE                        \ Keyboard 
    &R C, &1 C, &2 C, &3 C, &4 C,
    &5 C, &6 C, &7 C, &8 C, &9 C,
    &* C, &0 C, &# C,

FORTH DEFINITIONS

\G Wait until a key is pressed. Next leave the corresponding
\G character on the stack.
: AKEY          ( -- char )             \ Keyboard 
    BEGIN   AKEY?  20 MS                \ Key pushed down ?
            AKEY?  AND                  \ Are we shure ?
    UNTIL
    KEY-NR  KEY-TABLE + C@              \ Get ASCII value of key
    ;

PREVIOUS  FORTH

                       \ (* End of Source *) /
