\ ==============================================================
\ [TITLE]     8051 ANS-Forth LCD word set. 
\ [FILE]      LCD.FRT
\ [CATEGORY]  Example
\ [VSN]       1.00 
\ [CREATED]   July 17, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] August 18, 1998, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994-1998
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      CLOCK.FRT
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   LCD-display thru port P1 of the 80Cxxx
\             Use a display with the Hitachi HD44780A00H chip
\             Example of high level I/O port use, revectoring
\             of a system vector and connecting an LCD-display.
\ ==============================================================

ROM

MARKER -LCD                         \ Mark start of LCD code

ASSEMBLER
\ Make port P1: high level accessable. See also: SFR .
P1: SFR P1  ( -- byte )             \ Lcd  Access port P1 with TO concept
FORTH

\ Bit pattern which set the data valid (enable) bit.
$20 CONSTANT VALID  ( -- x )        \ Lcd  Data valid bit pattern

\ Bit pattern which set the character data bit
$40 CONSTANT CHARACTER ( -- x )     \ Lcd  Data is character bit pattern

: WAIT  500 0 DO LOOP ;             \ Lcd  Use delay loop when i/o to fast

\ Send one data nibble (4 bits) x to the LCD display. When flag
\ is true x is accepted as data. When flag is false x is accepted
\ as instruction.
: NIBBLE!   ( x flag -- )           \ Lcd 
  IF  CHARACTER OR THEN DUP TO P1   \ Send one nibble to an LCD-display
\ WAIT
  VALID  OR             DUP TO P1 
\ WAIT
  VALID  XOR                TO P1 
\ WAIT
  ;

\ Send one character char as two nibbles to an LCD-display. When
\ flag is true, char is accepted as data. When flag is false,
\ char is accepted as instruction.
: LCD!      ( char flag -- )        \ Lcd 
  >R                                \ Send byte as two nibbles to an LCD
   CSPLIT R@  NIBBLE!               \ High nibble first
          R>  NIBBLE!
  ;

\G Send one instruction byte to an LCD-display
: LCD-INSTR     ( byte -- )         \ Lcd 
  FALSE LCD!                        \ Mark data as instruction for an LCD
  ;

\G Send one character char to an LCD-display
: LCD-EMIT      ( char -- )         \ Lcd 
  TRUE LCD!                         \ Mark data as character
  ;

\G Initialise an LCD-display. Set 4 bit data bus, clear display,
\G select block cursor and display direction left to right.
: INIT-LCD      ( -- )              \ Lcd 
  $28 LCD-INSTR     2 MS            \ Set display to 4 bits I/O
  $02 LCD-INSTR     2 MS            \ Synchronize display with char. RAM
  $01 LCD-INSTR     2 MS            \ Clear display
  $06 LCD-INSTR                     \ Display direction is left to right
  $0D LCD-INSTR                     \ Block cursor on
  ;

\ Now some user functions

\G Clear LCD-display and position cursor in upper left position.
: LCD-PAGE      ( -- )              \ Lcd 
    1 LCD-INSTR  2 MS               \ Clean LCD and put cursor in upper left corner
    ;

\G Display the character string ccc at the LCD-display
: LCD(          ( "ccc" -- )        \ Lcd 
    PUSH EMIT                       \ Save original EMIT vector
    ['] LCD-EMIT IS EMIT            \ Set EMIT vector to the LCD
    POSTPONE .(                     \ Compile .(
    POP EMIT                        \ Restore original EMIT vector
    ;  IMMEDIATE

\G Display a 24-hour clock at the LCD-display
: LCD-CLOCK     ( -- )              \ Lcd 
    LCD-PAGE                        \ Clean LCD-display
    BEGIN
        $80 LCD-INSTR               \ Cursor to upper left corner
        PUSH EMIT                   \ Save original EMIT
        ['] LCD-EMIT  IS EMIT       \ Set EMIT to LCD-EMIT
        ." Current time is: "       \ Print a message
        .TIME                       \ Print time
        POP EMIT                    \ Restore EMIT
        STOP?                       \ Do until any key is pressed
    UNTIL
    ;

\G Display a chronometer at the LCD-display
: LCD-TIMER     ( -- )              \ Lcd 
    RESET-TIMER                     \ Set chronometer to zero
    LCD-PAGE                        \ Clean LCD
    BEGIN
        $80 LCD-INSTR               \ Cursor to upper left
        PUSH EMIT                   \ Save original EMIT
        ['] LCD-EMIT  IS EMIT       \ Set EMIT to LCD-EMIT
        ." Elapsed are: "           \ Print  a message
        .TIMER                      \ Print elapsed time
        POP EMIT                    \ Restore EMIT
        STOP?                       \ Do until any key is pressed
    UNTIL
    ;

\ ' BOOT AUTOSTART                  \ Autostart Forth interpreter
                                    \ to save LCD code for a reset

                    \ (* End of Source *) /
