\ ==============================================================
\ [TITLE]     8051 ANS Forth MENU word set. 
\ [FILE]      MENU1.FRT
\ [CATEGORY]  Example
\ [VSN]       1.01
\ [CREATED]   August 22, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] October 18, 1995, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994/1995
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Simple menu building word, inspired by a menu
\             demonstrated by Albert Nijhof. For beginners ;-)
\ ==============================================================

\G DOC  How do we use this menu structure
\G 
\G An example in pseudocode:
\G 
\G BEGIN-MENU
\G <xt1>   <string1>   <char1>
\G <xt2>   <string2>   <char2>
\G <xt3>   <string3>   <char3>
\G MENU <name>
\G 
\G <xt1> is the executing token that is executed if <char1> is
\G pressed when we are in the menu <name>. The strings <string1>
\G to <string3> are printed when the menu <name> is executed.
\G The name of the menu itself is printed in its header.
\G 
\G Data structure of this menu word. First the key character, then
\G the counted menu string and the execution token to be executed 
\G when the key character was pressed. At last the END-MENU token 
\G is compiled.
\G 
\G ---------------------------
\G |Char1|string pointer1|xt1|
\G ---------------------------
\G |Char2|string pointer2|xt2|
\G ---------------------------
\G |Char3|string pointer3|xt3|
\G ---------------------------
\G |End-menu|
\G ----------
\G 
\G ENDDOC

MARKER -MENUS  ( Menu building word vsn 1.01 )

\G Begin a new menu
        -1 CONSTANT BEGIN-MENU ( -- -1 ) \ Menu

\ End of menu symbol
BEGIN-MENU CONSTANT END-MENU   ( -- -1 ) \ Menu

\ Type a line of +n dashes
: .LINE         ( +n -- )               \ Menu  Print a line of n dashes long
    0 ?DO   [CHAR] - EMIT   LOOP
    ;

\ Type the menu from the data-field of which body is the begin 
\ address. Display a header, all menu strings and key characters.
: .MENU         ( body -- )             \ Menu  Print a menu
    PAGE CR 10 .LINE SPACE
    DUP BODY> >NAME .NAME
    10 .LINE
    CR CR ." Esc = Leave menu " CR
    BEGIN   @+ DUP END-MENU <>          \ Menu  End of menu reached ?
    WHILE   CR 4 SPACES EMIT ."  = "
            @+ COUNT TYPE
            CELL+
    REPEAT  2DROP
    CR CR 4 SPACES ." Choose "
    ;

\G Compile a string ccc as a counted string. Leave the address
\G a-addr on top of the stack.
: ""    ( "ccc" -- c-addr )             \ Menu Compile a string constant
    HERE  POSTPONE S"  PACK, ALIGN
    ;

\ Handle the menu from the data-field of which body is the begin
\ address. That is, display the menu strings and corresponding
\ key characters. Execute a token after the corresponding key is
\ pressed. This word will be left when the escape key is pressed.
\
\ Note: The inner loop uses a control-structure feature allowed by
\ the new ANSI Forth standard. This loop uses a multiple WHILE:
\ BEGIN .. WHILE .. WHILE .. REPEAT .. ELSE .. THEN 
\ The BEGIN and second WHILE are matched by REPEAT and the first
\ WHILE is matched by ELSE which matches with THEN . ANSI Forth
\ provides this very flexible control structures, but gives 
\ more responsibility to the programmer too.
: HANDLE-MENU   ( body -- )             \ Menu Handle a menu key and data 
    >R  R@ .MENU                        \ Print menu
    BEGIN   KEY >CAPITAL                \ Key always uppercase
            DUP ^[ <>                   \ Key is not Esc ?
    WHILE   R@  SWAP >R
            BEGIN   @+ DUP R@ <>        \ Key not equal to menu entry ?
            WHILE   END-MENU <>         \ Not at end of menu data ?
            WHILE   CELL+ CELL+
            REPEAT  R> 2DROP            \ Invalid key !
            ELSE    R> 2DROP            \ Valid key, execute token
                    CELL+ @ EXECUTE
                    R@ .MENU            \ Reprint menu
            THEN
    REPEAT  R> 2DROP
    ;

\G Compiling: ( <begin> xt1 c-addr1 char1 ... xtn c-addrn charn "name" -- )
\G Skip leading space delimiters. Parse name delimited by a space
\G Create a definition for name with the execution semantics 
\G defined below. Compile a menu with 1 to n entries.
\G
\G Executing: ( i*x -- j*x )
\G Start and display a menu, execute an xt if the corresponding
\G key is pressed. After executing xt return to the menu and redraw
\G it. The menu will be left when the escape key is pressed.
: MENU                                  \ Menu Define a menu
    CREATE              ( i*x "name"  -- )
        BEGIN   DUP BEGIN-MENU <>
        WHILE   ,  ,  ,
        REPEAT  ,
    DOES> HANDLE-MENU   ( i*x -- j*x )
    ;

( Menu structure loaded, adding a test now )

: HANG      KEY DROP ;
: .ORDER    CR ORDER HANG ;
: .WORDS    CR WORDS HANG ;
: .WLIST    CR CR ." Wordlists " .WORDLISTS HANG ;

BEGIN-MENU
    ' .WORDS        "" Show current word list"      &W
    ' .ORDER        "" Display search order"        &S
    ' .WLIST        "" Display all vocabularies"    &V
MENU SYSTEM-INFO

BEGIN-MENU
    ' SYSTEM-INFO   "" Start INFO menu"             &I
    ' .WLIST        "" Display all vocabularies"    &V
MENU MAIN
