\ ==============================================================
\ [TITLE]     8052-ANS-Forth making music with one bit. 
\ [FILE]      MUSIC.FRT
\ [CATEGORY]  Demo
\ [VSN]       1.00 
\ [CREATED]   March 8, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] August 19, 1998, Willem Ouwerkerk 
\ [COPYRIGHT] De Elektrise Schaar (C) 1995-1998
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8052-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Using timer 1 to create Pulses at P1.5
\ ==============================================================
\ Parts:        BS170, resistor 47 KOhm, 47 Ohm and a 50 Ohm speaker.
\               Timing loops are valid for a standard 80C32/52 at 12 MHz

\ Connect a driver and small speaker thru output P1.5

?DEF -MUSIC [IF]  TF0-VEC CLEAR-VECTOR  -MUSIC  [THEN]  MARKER -MUSIC

$40 SFR NOTE-TIME                       \ Length of note timer
$41 SFR TEMPO                           \ Length of a note

ASSEMBLER
P1: 5 + CONSTANT LS
$88 CONSTANT TCON:
$89 CONSTANT TMOD:
$8A CONSTANT TL0:
$8C CONSTANT TH0:
$A8 CONSTANT IEN0:
FORTH

CODE SETUP-MUSIC    ( -- )              \ Install timer 0
    TMOD: $F0 # ANL,                    \ Reset timer 0, leave timer 1
    TMOD: $01 # ORL,                    \ Timer 0 to mode 1 !!
    TH0: -5000 >< # MOV,                \ Set count to 1 ms
    TL0: -5000 # MOV,
    IEN0: .1 SETB,                      \ Timer 0 interrupt on
    IEN0: .7 SETB,                      \ Allow all interrupts
    TCON: .4 SETB,                      \ Start timer 0
    RET,
END-CODE

CODE TIMER0         ( -- )              \ Interrupt code
    TH0: -5000 >< # MOV,                \ Reload timer 0
    TL0: -5000 # MOV,
    $40 INC,                            \ Advance note time
    RETI,
END-CODE

CODE PERIOD         ( -- )              \ Delay 15 us to 3840 us (1-256)
    R4: [R2] MOV,
    DO,     AB: MUL,  AB: MUL,
            AB: MUL,  NOP,
\           AB: MUL,  AB: MUL,
\           AB: MUL,  AB: MUL,
\           AB: MUL,  AB: MUL,
    R4: LOOP,
    RET,
END-CODE

\ If the smallest period is 15 us, a notes period is:
\
\                 33333
\ Period = --------------------
\           Freq of note in Hz

CODE SQUARE         ( period -- )       \ Squarewave with a period time of
    R0: INC,                            \ period * 15 us
    A: @R0 MOVX,
    R2: A: MOV,
    R0: INC,
    $40 0 # MOV,
    BEGIN,  LS SETB,                    \ LoudSpeaker = 1
            ' PERIOD CALL,
            LS CLR,                     \ LoudSpeaker = 0
            ' PERIOD CALL,
            C: SETB,
            A: $41 MOV,                 \ Loop until TEMPO < NOTE-TIME
            A: $40 SUBB,
    CS UNTIL,
    RET,
END-CODE

\ : SQUARE            ( period -- )     \ ??
\    SQUARE  TEMPO 0 ?DO  7 0 DO LOOP  LOOP
\    ;

\G Wait for one note length.
: REST              ( -- )              \ Music  Silence for TEMPO millisec.
    TEMPO 10 *  MS 
    ;

\G Set full note length. Other note lengths are: /2 /4 /8 
\G /12 /16 /32.
: /1        250 TO TEMPO ;  : /2        125 TO TEMPO ;  ( -- ) \ Music

: /4         63 TO TEMPO ;  : /8         31 TO TEMPO ;
: /12        21 TO TEMPO ;  : /16        16 TO TEMPO ;
: /32         8 TO TEMPO ;

\G Play note F1 with current note length. Other notes 
\G are: FIS1 to E3.
: F1        189 SQUARE ;    : FIS1      179 SQUARE ;  ( -- ) \ Music

: G1        169 SQUARE ;    : GIS1      159 SQUARE ;
: A1        150 SQUARE ;    : AIS1      142 SQUARE ;
: B1        134 SQUARE ;    : C1        127 SQUARE ;    
: CIS1      120 SQUARE ;    : D1        113 SQUARE ;
: DIS1      107 SQUARE ;    : E1        101 SQUARE ;    

: F2         96 SQUARE ;    : FIS2       90 SQUARE ;
: G2         85 SQUARE ;    : GIS2       80 SQUARE ;    
: A2         76 SQUARE ;    : AIS2       72 SQUARE ;    
: B2         67 SQUARE ;    : C2         64 SQUARE ;    
: CIS2       60 SQUARE ;    : D2         57 SQUARE ;
: DIS2       54 SQUARE ;    : E2         51 SQUARE ;    

: F3         48 SQUARE ;    : FIS3       45 SQUARE ;
: G3         43 SQUARE ;    : GIS3       40 SQUARE ;
: A3         38 SQUARE ;    : AIS3       36 SQUARE ;
: B3         34 SQUARE ;    : C3         32 SQUARE ;
: CIS3       30 SQUARE ;    : D3         28 SQUARE ;
: DIS3       27 SQUARE ;    : E3         25 SQUARE ;    

\ Speaker octave demo .....

: OCTAVE            ( -- )              \ Demo word
    REST F2 FIS2 G2 GIS2 A2 AIS2 B2 C2 CIS2 D2 DIS2 E2 REST
    ;

: MUSIC-DEMO        ( -- )              \ Play some notes
    ['] TIMER0 TF0-VEC SET-VECTOR       \ Install timer 0 interrupt
    SETUP-MUSIC

    /16 OCTAVE      /8 OCTAVE
     /4 OCTAVE       /2 OCTAVE

    /4 REST A1  REST A2  REST A3
       REST C1  REST C2  REST C3
    ;

' TIMER0    TF0-VEC     SET-VECTOR      \ Install timer 0 interrupt
' MUSIC-DEMO  AUTOSTART                 \ Perform music demo on startup

\ End
