\ ==============================================================
\ [TITLE]     8051 ANS-Forth OPT-CODE word set. 
\ [FILE]      OPT-CODE.FRT
\ [CATEGORY]  Example
\ [VSN]       1.10 
\ [CREATED]   May 27, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] May 30, 1994, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] Optimising compiler
\ [USES]      8051-ANS-Forth internal knowledge 
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Runtime optimizer code (Highlevel example)
\ ==============================================================

INTERNAL DEFINITIONS

: LIT+      ( n1 inline# -- n2 )
    R> DUP CELL+ >R @ +
    ;  TAIL

: LIT-      ( n1 inline# -- n2 )
    R> DUP CELL+ >R @ -
    ;  TAIL

: LIT=      ( n1 inline# -- fl )
    R> DUP CELL+ >R @ +
    ;  TAIL

: LITAND    ( x1 inline# -- x2 )
    R> DUP CELL+ >R @ AND
    ;  TAIL

: LITOR     ( x1 inline# -- x2 )
    R> DUP CELL+ >R @ OR 
    ;  TAIL


: EXT-SIGN  ( byte -- n )               \ The 8051 ANS-Forth has signed byte
    DUP $80 AND IF  $FF00 OR  THEN      \ literals, so we must extend the sign.
    ;

: CLIT+     ( n1 inline# -- n2 )
    R> DUP 1+ >R C@ EXT-SIGN +
    ;  TAIL

: CLIT-     ( n1 inline# -- n2 )
    R> DUP 1+ >R C@ EXT-SIGN -
    ;  TAIL

: CLIT=     ( n1 inline# -- fl )
    R> DUP 1+ >R C@ EXT-SIGN +
    ;  TAIL

: CLITAND   ( x1 inline# -- x2 )
    R> DUP 1+ >R C@ EXT-SIGN AND
    ;  TAIL

: CLITOR    ( x1 inline# -- x2 )
    R> DUP 1+ >R C@ EXT-SIGN OR 
    ;  TAIL


: OVER+     ( n1 n2 -- n1 n3 )
    OVER +
    ;

: OVER-     ( n1 n2 -- n1 n3 )
    OVER -
    ;

: OVER=     ( n1 n2 -- n1 n3 )
    OVER =
    ;

: OVERAND   ( n1 n2 -- n1 n3 )
    OVER AND
    ;

: OVEROR    ( n1 n2 -- n1 n3 )
    OVER OR 
    ;

: OVER@     ( n1 n2 -- n1 n3 )
    OVER @
    ;

: OVERC@    ( n1 n2 -- n1 n3 )
    OVER C@
    ;


: DUP@      ( addr -- addr x )
    DUP @
    ;

: DUPC@     ( addr -- addr byte )
    DUP C@
    ;

: DUP0=     ( addr -- addr flag )
    DUP 0=
    ;

: ?DUP0=    ( n -- n false | true )
    ?DUP 0=
    ;


: 3DROP     ( n1 n2 n3 -- )
    DROP 2DROP
    ;

: RDROP     ( R: x -- )
    2R> NIP  >R
    ;  TAIL


: I+        ( n1 -- n2 )
    R> I  SWAP >R  +
    ;

: I+C@      ( n1 -- n2 )
    R> I  SWAP >R  + C@
    ;


: TBRANCH   ( flag -- )
    IF      R> @ >R
    ELSE    R> CELL+ >R
    THEN
    ;  IMMEDIATE  TAIL

FORTH DEFINITIONS

                    \ (* End of Source *) /
