\ ==============================================================
\ [TITLE]     8051 ANS-Forth exetend optimiser words set. 
\ [FILE]      OPT-EXT.FRT
\ [CATEGORY]  Example
\ [VSN]       2.10 
\ [CREATED]   May 13, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] May 31, 1994, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] Optimising compiler
\ [USES]      8051-ANS-Forth internal knowledge
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Extend Forth with special optimising primitives
\ ==============================================================

\ ---- This is mostly 8051 ANS-Forth specific code -------------
\ ---- Optimising exit (tail call removal) ---------------------

INTERNAL ALSO  FORTH DEFINITIONS

\ ---- Control structure words with optimiser ------------------

-1
ASS" ' 0= LCALL,"   0   ' TBRANCH       \ 0BRANCH ... Branch if true
OPTIMISER 0BRANCH  C/O

: IF        ( -- addr n )       \ Redefine an optimising IF to IFZ
    POSTPONE 0BRANCH            \ Compile optimising 0BRANCH
    HERE  0 ,  1                \ Compile destination
    ;  IMMEDIATE  C/O

: UNTIL     ( addr n -- )
    4 ?PAIRS                    \ Check structure
    POSTPONE 0BRANCH  ,         \ Compile optimising 0BRANCH and branch dest.
    ;  IMMEDIATE

: WHILE     ( addr1 n1 -- addr2 n2 addr1 n1 )
    [COMPILE] IF  2SWAP         \ Reuse IF
    ;  IMMEDIATE


\ ---- Words with embedded optimiser ---------------------------
\ These are the sequences to optimise for 8051 ANS-Forth kernel

-1
ASS" ' LIT  LCALL,"     0   ' LIT+      \ +
ASS" ' CLIT LCALL,"     0   ' CLIT+
ASS" ' OVER LCALL,"     0   ' OVER+
ASS" ' I    LCALL,"     0   ' I+
OPTIMISER +

-1
ASS" ' LIT  LCALL,"     0   ' LIT-      \ -
ASS" ' CLIT LCALL,"     0   ' CLIT-
ASS" ' OVER LCALL,"     0   ' OVER-
OPTIMISER -

-1
ASS" ' LIT  LCALL,"     0   ' LITAND    \ AND
ASS" ' CLIT LCALL,"     0   ' CLITAND
ASS" ' OVER LCALL,"     0   ' OVERAND
OPTIMISER AND
 
-1
ASS" ' LIT  LCALL,"     0   ' LITOR     \ OR
ASS" ' CLIT LCALL,"     0   ' CLITOR
ASS" ' OVER LCALL,"     0   ' OVEROR
OPTIMISER OR

-1
ASS" ' LIT  LCALL,"     0   ' LIT=      \ =
ASS" ' CLIT LCALL,"     0   ' CLIT=
ASS" ' OVER LCALL,"     0   ' OVER=
OPTIMISER =

-1
ASS" ' DUP  LCALL,"     0   ' DUP0=     \ 0=
ASS" ' ?DUP LCALL,"     0   ' ?DUP0=
OPTIMISER 0=

-1
ASS" ' DROP   LCALL,"   0   ' 2DROP     \ DROP
ASS" ' 2DROP  LCALL,"   0   ' 3DROP
ASS" ' SWAP   LCALL,"   0   ' NIP
ASS" ' DODOES LCALL,"   0   ' RDROP
ASS" ' R>     LCALL,"   1   :MACRO      \ Assemble POP a word
                                SP: DEC,  SP: DEC,
                            ENDM
OPTIMISER DROP

-1
ASS" ' LIT  LCALL,"     0   ' (FROM)    \ @  (FROM) is LIT@
ASS" ' CLIT LCALL,"     1   :MACRO
                                ['] (FROM) LCALL,
                                HERE C@ ,
                            ENDM
ASS" ' OVER LCALL,"     0   ' OVER@
ASS" ' DUP  LCALL,"     0   ' DUP@
OPTIMISER @

-1
ASS" ' I+   LCALL,"     0   ' I+C@      \ C@
ASS" ' OVER LCALL,"     0   ' OVERC@
ASS" ' DUP  LCALL,"     0   ' DUPC@
OPTIMISER C@

-1
ASS" ' SWAP LCALL,"     0   ' TUCK      \ OVER
OPTIMISER OVER

ONLY  EXTRA ALSO  FORTH ALSO  DEFINITIONS

                          \ (* End of Source *) /
