\ ==============================================================
\ [TITLE]     8052-ANS-Forth RC5 woorden. 
\ [FILE]      RC5.FRT
\ [CATEGORY]  Tools
\ [VSN]       1.00 
\ [CREATED]   July 27, 1995, Willem Ouwerkerk 
\ [LASTCHNGD] August 16, 1998, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1995-1998
\ [MODULE OF] 8052-ANS-Forth 
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Decoding of RC5 remote control signals
\ ==============================================================

?DEF -RC5 [IF]  -RC5  [THEN]  MARKER -RC5

ASSEMBLER ALSO  DEFINITIONS             \ Add needed asm labels

    $89 CONSTANT TMOD:
    $88 CONSTANT TCON:
    $A8 CONSTANT IE:                    \ Ok for 80C32 and 80C535
    $8B CONSTANT TL1:
    $8D CONSTANT TH1:

\ == RC5 decoder ===============================================

PREVIOUS  INTERNAL ALSO DEFINITIONS

#05 CONSTANT RC-TYPE                    \ RC5 Set transmitter type
                                        \ TV = 0, CD = 20 and VCR = 5
$40 SFR RC-SYS                          \ RC5 system databyte (8 bits)
$41 SFR RC-DATA                         \ RC5 command databyte (6 bits)
$42 SFR RC#                             \ RC5 received bits count

EXTRA ALSO DEFINITIONS

\G If an RC5 command is avialable from the selected transmitter,
\G return true. Otherwise false. See also: RC-KEY
FLAG RC-KEY?                            \ RC5 "r-c-key-question" command received

INTERNAL DEFINITIONS

FLAG RC-BIT                             \ RC5 first half bit, for control
$B2 BIT-SFR RC-IN                       \ RC5 input bit (port 3.2)

CODE RC-SETUP      ( -- )               \ Setup for RC5 receiving
( ) TMOD: $10 # ORL,                    \ Set Timer 1 in mode 1
    TCON: .6 CLR,                       \ Stop timer 1
    IE: .3 SETB,                        \ Enable timer 1 interrupts
    TCON: .0 SETB,                      \ EX0 active on falling edges
    IE: .0 SETB,                        \ EX0 interrupt on
    ADR RC-KEY? CLR,                    \ No RC5 data received
    ADR RC-IN SETB,                     \ P3.2 is input
( ) IE: .7 SETB,                        \ Global enable interupts
    RET,
END-CODE


\ RC5 remote control receiver (code for an 12 MHz xtal).

CODE RC-START      ( -- )               \ RC5 start receiving 
    PSW: PUSH,
    ADR RC-KEY? BC IF,                  \ RC5 data processed ?
        IE: .0 CLR,                     \ Yes, EX0 interrupt off
        TH1: -400 >< # MOV,             \ Set negated 16 bits count
        TL1: -400 # MOV,
        TCON: .6 SETB,                  \ Start timer 1
        ADR RC# 1 # MOV,                \ Half bit received
        ADR RC-BIT SETB,                \ First half bit is high!
    THEN,
    PSW: POP,
    RETI,
END-CODE

CODE SAVE-RC-BIT    ( -- )              \ Save received bit
    A: 16 # ANL, 
    ZER IF,                             \ Is it a system bit ?
        A: ADR RC-SYS MOV,              \ Yes
        A: RLC,
        ADR RC-SYS A: MOV,
    ELSE,                               \ No, its a data bit
        A: ADR RC-DATA MOV,
        A: RLC,
        ADR RC-DATA A: MOV,
    THEN,
    ADR RC# INC,                        \ Increase bit counter
    RET,
END-CODE

CODE RECEIVE        ( -- )              \ RC5 read bit routine
    PSW: PUSH,
    ACC: PUSH,
    TH1: -880 >< # MOV,                 \ Set negated 16 bits count
    TL1: -880 # MOV,
    C: ADR RC-IN MOV,                   \ Read next RC5 bit
    A: ADR RC# MOV,
    ACC: .0 BS IF,                      \ Collect every second bit ?
        ' SAVE-RC-BIT CALL,             \ Save a bit
        A: CLR,                         \ Check bit pattern !
        C: ADR RC-BIT MOV,
        A: RLC,
        C: ADR RC-IN MOV,
        A: 0 # ADDC,
        A: 1 # EQL IF,                  \ Valid pattern 0/1 of 1/0 received ?
            A: ADR RC# MOV,             \ End of bitstream
            A: $1C # EQL IF,            \ 14 bits received ?
                ADR RC-SYS $FF # XRL,   \ Invert system bits
                ADR RC-DATA $FF # XRL,  \ Invert data bits
                ADR RC-DATA $3F # ANL,  \ Leave only lower 6 data bits
                A: ADR RC-SYS MOV,
                A: $1F # ANL,           \ Leave transmitter ID
                A: RC-TYPE # EQL IF,    \ Is it a .... remote control ?
                    ADR RC-KEY? SETB,   \ Yes, new data word received !
                    IE: .0 SETB,        \ EX0 interrupt on
                    TCON: .6 CLR,       \ Stop timer 1
                THEN,
            THEN,
        ELSE,
            IE: .0 SETB,                \ EX0 interrupt on
            TCON: .6 CLR,               \ Stop timer 1
        THEN,
    ELSE,
        ADR RC-BIT C: MOV,              \ Save half bit value
        ADR RC# INC,                    \ Increase bit count
    THEN,
    ACC: POP,
    PSW: POP,
    RETI,
END-CODE

EXTRA DEFINITIONS

\G Receive one command byte from the RC5 commandcode set and the
\G selected transmitter. If command is not available or not valid
\G wait until a valid command is received. See also: RCKEY?
: RC-KEY        ( -- byte )             \ RC5 "r-c-key" Get command databyte
    BEGIN  RC-KEY? UNTIL                \ Wait
    RC-DATA                             \ Push command
    CLEAR RC-KEY?                       \ Make way
    ;

INTERNAL DEFINITIONS

\ Install interrupt routines and demo

: STARTUP      ( -- )
    ['] RECEIVE  TF1-VEC  SET-VECTOR    \ Install timer 1 interrupt
    ['] RC-START IE0-VEC  SET-VECTOR    \ Install ext. interrupt 0
    RC-SETUP  BOOT                      \ Install registers and boot Forth
    ;

\ Einde van RC5 decoder

' STARTUP  AUTOSTART                    \ Start RC5 decoder

FRESH  DEFINITIONS

\ Einde
