\ ==============================================================
\ [TITLE]     8051 ANS-Forth strings word set. 
\ [FILE]      strings.FRT
\ [CATEGORY]  Example
\ [VSN]       1.00 
\ [CREATED]   ????, Albert Nijhof
\ [LASTCHNGD] October 19, 1994, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   8051-ANS-Forth string package with methods mechanism
\             and 8051 assembler and DOER:
\ ==============================================================

MARKER -STRING ( Original Albert Nijhof adapted by Willem Ouwerkerk )

FALSE [IF]

: C+!       ( n c-addr -- )             \ Length 22 bytes + header
    TUCK C@ +  SWAP C!
    ;

[ELSE]

\G Add n to the contents of c-addr
CODE C+!    ( n c-addr -- )             \ Strings  Length: 23 bytes + header
    A: @R0 MOVX,                        \ Pop high byte c-addr
    DPH: A: MOV,
    R0: INC,
    
    A: @R0 MOVX,                        \ Pop low byte c-addr
    DPL: A: MOV,
    R0: INC,
    
    R0: INC,                            \ Pop high byte of n
    A: @R0 MOVX,                        \ Pop and save n
    R1: A: MOV,
    R0: INC,

    A: @DPTR MOVX,                      \ Fetch contents of c-addr
    A: R1: ADD,                         \ Add n to it
    @DPTR A: MOVX,                      \ Store result in c-addr

    RET,    
END-CODE

[THEN]

\ -- First the operations --

INTERNAL ALSO  DEFINITIONS
 
: TO$       ( c-addr +n inlineadr -- )  \ Store a stringvariable
  INLINE#
  COUNT ROT UMIN
  SWAP PLACE
  ;

\ Stringvar. to stack, <adr> and <+n> depend on <offset> and <n>
 
: FROM$     ( offset n inlineadr -- adr +n )
  INLINE#  1+ COUNT                     \ Get string address and length
  LOCALS| LEN ADR N OFFS |              \ Put all data in locals
    
  OFFS 0<                               \ Offset negative ?
  IF    LEN +TO OFFS                    \ Yes, add string length to offset
  THEN  OFFS  DUP N +                   \ Offset  and offset + string length
  N 0< IF  SWAP  THEN                   \ Change order when n is negative
  
  0 MAX LEN UMIN  >R                    \ Adjust 1
  0 MAX LEN UMIN                        \ Adjust 2
  DUP ADR +  R> ROT -                   \ Leave wanted string part
  ;
 
: +TO$      ( a +n inlineadr -- )       \ Extend stringvar. with <a> <+n>
  INLINE#
  COUNT OVER C@ -  OVER >R
  ROT UMIN >R
  COUNT + R@
  MOVE  R> R> C+!
  ;
 
: ADD$      ( char inlineadr -- )       \ Add char behind stringvar.
  INLINE#
  COUNT OVER C@ >                       \ String not full ?
  IF
    DUP >R
    COUNT + C!
    1 R> C+!  EXIT
  THEN
  2DROP
  ;
 
DOER: DO$VAL  ( -- c-addr u )           \ Push string var.
  1+  COUNT
  ;

METHODS DO$VAL                          \ Add methods to $VARIABLE

: FROM        ( offset n "name" -- c-addr u ) ( body -- C)
    POSTPONE FROM$  ,
    ;  IMMEDIATE
    
: TO          ( c-addr +n "name" -- ) ( body -- C)
    POSTPONE TO$    ,
    ;  IMMEDIATE

: +TO         ( c-addr +n "name" -- ) ( body -- C)
    POSTPONE +TO$   ,
    ;  IMMEDIATE

: ADD         ( char "name" -- ) ( body -- C)
    POSTPONE ADD$   ,
    ;  IMMEDIATE

END-METHODS

2 MESS" String to long, max 255 chars"  \ The creating warning

 
\ -- Nu de stringvariabele bouwer --

FORTH DEFINITIONS

\G ( +n "name" -- ) 
\G Skip leading space delimiters. Parse name delimited by a space
\G Create a definition for name with the execution semantics 
\G defined below. Reserve +n+1 bytes with a empty counted string.
\G
\G name Execution: ( -- c-addr u )
\G Return c-addr u, describing the string in its data space. The
\G string will be replaced when using TO on name or extended when
\G using +TO on name. A partial string will be returned when the
\G FROM prefix is used at name, where offset and n decribe this 
\G string.
: $VALUE                                \ Strings  Define empty stringvar of <+n> chars
  $FF OVER U< 2 ?ERROR                  \ Length does not includes count
  CREATE
    DUP C,  0 C,  ALLOT ALIGN IMMEDIATE \ Reserve string space
  DO$VAL
  ;

PREFIX ADD                              \ Prefix to add a char. to a string

PREVIOUS  FORTH
 
\ Examples:
\ #16 $VALUE STRING                     ( Define string )
\ S" Stringetje " TO STRING             ( Fill string )
\ 2 4 FROM STRING TYPE                  ( Typed: "ring" )
\ -2 4 FROM STRING TYPE                 ( Typed: "e " )
\ 2 -4 FROM STRING TYPE                 ( Typed: "St" )
\ -2 -4 FROM STRING TYPE                ( Typed: "getj" )
\ STRING TYPE                           ( Typed: "Stringetje " )
\ CHAR ! ADD STRING                     ( Add char "!" behind string )
\ STRING TYPE                           ( Typed: "Stringetje !" )
\ S" ? " +TO STRING                     ( Add "? " to string )
\ STRING TYPE                           ( Typed: "Stringetje !? " )
\ 5 3 from string to string             ( Get part of string and put it back )
\ STRING TYPE                           ( Typed: "get" )

                    \ (* End of Source *) /
