\ ==============================================================
\ [TITLE]     8051 ANS Forth VECTOR reference word set. 
\ [FILE]      VECTOR.FRT
\ [CATEGORY]  Example
\ [VSN]       1.00 
\ [CREATED]   January 1, 1994, Willem Ouwerkerk 
\ [LASTCHNGD] April 3, 1994, Willem Ouwerkerk 
\ [COPYRIGHT] Dutch Forth Users Group (C) 1994
\ [COMPILE]   CursusForth Environments
\ [MODULE OF] 8051-ANS-Forth 
\ [USES]      ...
\ [AUTHOR]    Willem Ouwerkerk 
\ [PURPOSE]   Example of how to build and handle execution vectors
\             The use of CREATE DOER: IS-FORGET :NONAME .
\ ==============================================================

MARKER -VECTOR                          \ Mark the vectored definitions part

INTERNAL  ALSO DEFINITIONS

DOER: DOVEC     ( body -- )
    @ EXECUTE                           \ Get <xt> from vector and execute.
    ;

EXTRA DEFINITIONS

\G ( "name" -- )
\G Skip leading space delimiters. Parse name delimited by a space.
\G Create a new definition for name with the execution semantics
\G below.
\G
\G name Execution: ( i*x -- j*x )
\G Execute the token from the datafield of name. By default this
\G token is the token that represents ABORT . This token may be
\G replaced using the words LET and DO:
: VECTOR                                \ Vector
    CREATE  ['] ABORT ,  DOVEC          \ Build vector "name" with action ABORT
    ;

\G Skip leading space delimiters. Parse name delimited by a space.
\G Locate the data field of a vector include its address in vector-
\G sys for DO:. See the example in VECTOR.FRT
\G See also: VECTOR .
: LET           ( "name" -- vectorsys ) \ Vector
    FROM >IN >R                         \ Save input pointer
    '  DUP 1+ @                         \ Get <xt> of "name" and exec. addr
    DUP  [ ' DOVEC 3 + 3 CELLS +  ] LITERAL  <> \ Is it not a vector
    SWAP [ ' DOUVEC 3 + 3 CELLS + ] LITERAL  <> \ Is it not a user vector
    TUCK  AND -70 ?ERROR                \ Non of both (no vector)
    IF      >BODY                       \ It's a vector get bodyaddress
    ELSE    DROP                        \ Remove <xt>
            R@ TO >IN                   \ Restore input pointer
            POSTPONE ADR                \ Get address of user vector
    THEN
    R> DROP  20                         \ Remove inp. pointer & syntax nr.
    ;

INTERNAL DEFINITIONS

: F:DO          ( body -- )
    DUP @ @  OVER 2 CELLS +  =          \ Is this routine installed ?
    IF      DUP CELL+ @  OVER @  !      \ Yes, restore it with old routine.
    THEN
    DROP                                \ Remove body address
    ;

DOER: DODO      ( body -- )
    DUP 2 CELLS +  SWAP @  !            \ Install my routine
    ;

' F:DO  IS-FORGET DODO                  \ Install my forget routine

EXTRA DEFINITIONS

\G ( vector-sys "name" -- )
\G Skip leading space delimiters. Parse name delimited by a space.
\G Create a new definition for name with the execution semantics
\G below. Describe an action for the vector named behind LET .
\G
\G name Execution: ( -- )
\G Install the token of the action defined with name in its
\G associated execution vector. See also: LET and VECTOR .
: DO:                                   \ Vector
    20 ?PAIRS                           \ Check syntax
    CREATE                              \ Build header
    DUP , @ ,                           \ Hold vector address and contents
    DODO                                \ Install installation routine
    :NONAME DROP  !CSP                  \ Build high level action
    ;

\ Test the VECTOR definition and it's handling routines

VECTOR APE

LET APE     DO: CHIMP  ." Chimpanzee " ;    CHIMP
LET APE     DO: GOR    ." Gorilla " ;       GOR
LET APE     DO: LAZY   ." Sloth " ;         LAZY

CHIMP   APE
GOR     APE
LAZY    APE

PREVIOUS  FORTH DEFINITIONS

                       \ (* End of Source *) /


